import lib.common.os
import lib.xuyuxing.base.base_function as bf
from lib.common.genome.seq_base import read_fasta_big, sub_seq
from lib.common.fileIO import tsv_file_dict_parse
from lib.common.math.interval import section, group_by_intervals
from config import *
from collections import Counter
import re
import os
import pysam


def get_lr(brax):
    hash = {}
    lefts = []
    i = 0
    for ch in brax:
        i = i + 1
        if ch == "(":
            lefts.append(i)
        elif ch == ")":
            left = lefts.pop()
            hash[left] = i
    return hash


def get_rl(brax):
    lr = get_lr(brax)
    rl = {}
    for left in lr:
        right = lr[left]
        rl[right] = left
    return rl


def check_mir_star(brax, strand, fold_start, fold_stop, mirkey_start, mirlen):
    if strand == "+":
        offset = mirkey_start - fold_start
    else:
        offset = fold_stop - mirkey_start - mirlen + 1

    if not (offset + mirlen) < len(brax):
        return ("N10",)

    mir_brax = brax[offset:offset + mirlen]
    n_mir_up = mir_brax.count(".")

    # print(n_mir_up)

    if n_mir_up > 5:
        return ("N11",)

    n_bulges = 0
    bulged_nts = 0
    last_right = 0
    last_left = 0

    if re.match(r'^[\.\(]+$', mir_brax):
        # find star of a 5p miRNA
        left_right = get_lr(brax)

        # get one-based start and stop of the mature miRNA, relative to brax
        mir_brax_start = offset + 1
        mir_brax_end = mir_brax_start + mirlen - 1

        # March through the mature miRNA until the last 2 bases (3' overhang),
        # tracking bulges, and getting the star 5p and 3p
        for left in range(mir_brax_start, mir_brax_end - 2):
            if left in left_right:
                if last_right and last_left:  # was it a bulge?
                    left_delta = left - last_left
                    right_delta = last_right - left_right[left]
                    if abs(left_delta - right_delta):
                        n_bulges = n_bulges + 1
                        bulged_nts = bulged_nts + abs(left_delta - right_delta)
                    else:
                        # This is the first pair found in the duplex
                        # Infer the 3p end of the miRNA* .. 2nt offset
                        star_brax_end = left_right[left] + 2 + (left - mir_brax_start)
                last_left = left
                last_right = left_right[left]
        if n_bulges > 2 or bulged_nts > 3:
            return ("N13",)
        else:
            if not last_left:
                return ("N10",)
            # Calculate the star 5p position, relative to brax
            star_brax_start = left_right[last_left] - ((mir_brax_end - 2) - last_left)
            # above, the right-hand term is 0 if the last position analyzed of the mature miRNA was paired.
        ## encode and return (below)
    elif re.match(r'^[\.\)]+$', mir_brax):
        # find star of a 3p miRNA
        right_left = get_rl(brax)
        # get one-based start and stop of the mature miRNA, relative to brax
        mir_brax_start = offset + 1
        mir_brax_end = mir_brax_start + mirlen - 1

        # March through the mature miRNA until the last 2 bases (3' overhang),
        # tracking bulges, and getting the star 5p and 3p
        for right in range(mir_brax_start, mir_brax_end - 2):
            if right in right_left:
                if last_right and last_left:
                    # was it a bulge?
                    left_delta = right_left[right] - last_left
                    right_delta = last_right - right
                    if abs(left_delta - right_delta):
                        n_bulges = n_bulges + 1
                        bulged_nts = bulged_nts + abs(left_delta - right_delta)
                else:
                    # This is the first pair found in the duplex
                    # Infer the 3p end of the miRNA* .. 2nt offset
                    star_brax_end = right_left[right] + (right - mir_brax_start) + 2
                last_left = right_left[right]
                last_right = right
        if n_bulges > 2 or bulged_nts > 3:
            return ("N13",)
        else:
            if not last_right:
                return ("N10",)
            # Calculate the star 5p position, relative to brax
            star_brax_start = right_left[last_right] - ((mir_brax_end - 2) - last_right)
    else:
        return ("N12",)

    if star_brax_start and star_brax_end:
        if star_brax_start >= star_brax_end:
            return ("N10",)
        else:
            if strand == "+":
                star_left = star_brax_start + fold_start - 1
            else:
                star_left = fold_stop - star_brax_end + 1
            mdz = star_brax_end - star_brax_start + 1
            return star_left, strand, mdz
    else:
        return ("N10",)


def mistar_get(mirkey_start, mirlen, refseq, strand, local_start, local_end, foldsize, mir_seq=""):
    loc_center = int(local_start) + int(0.5 * (int(local_end) - int(local_start) + 1))
    fold_start = max(1, loc_center - (int(0.5 * foldsize)))
    fold_stop = min(loc_center + (int(0.5 * foldsize)), len(refseq))

    fold_seq = sub_seq(refseq, fold_start, fold_stop, strand, True)

    cmd_string = "echo %s | %s --noPS " % (fold_seq, RNAfold_path)
    flag_tmp, output, error = lib.common.os.cmd_run(cmd_string, silence=True)
    output = output.decode()
    brax = output.split("\n")[1].split(" ")[0]

    check_mir_star_output = check_mir_star(brax, strand, fold_start, fold_stop, mirkey_start, mirlen)

    if len(check_mir_star_output) == 1:
        return check_mir_star_output

    mirkey_end = mirkey_start + mirlen - 1
    if mir_seq == "":
        mir_seq = sub_seq(refseq, mirkey_start, mirkey_end, strand, True)

    star_left, strand, mdz = check_mir_star_output
    mistar_start = star_left
    mistar_end = star_left + mdz - 1
    mistar_seq = sub_seq(refseq, mistar_start, mistar_end, strand, True)

    return (mirkey_start, mirkey_end, mir_seq, mistar_start, mistar_end, mistar_seq)


def blast_and_hit_parse(query_file, db_file, bls_file, aln_ratio, min_len, foldsize, num_threads):
    # load query fasta file
    record_dict = {}
    for record in read_fasta_big(query_file):
        record_dict[record.seqname_short()] = record

    # load db fasta file
    record_db_dict = {}
    for record in read_fasta_big(db_file):
        record_db_dict[record.seqname_short()] = record

    # make blast
    cmd_string = blast_path + "blastn -query %s -db %s -out %s -word_size 4 -gapopen 5 -gapextend 2 -reward 1 -penalty -3 -evalue 10 -outfmt 6 -num_threads %d" % (
        query_file, db_file, bls_file, num_threads)
    lib.common.os.cmd_run(cmd_string, silence=True)

    # load blast results
    outfmt6_title = (
        "query", "subject", "identity", "aln_len", "miss", "gap", "qstart", "qend", "sstart", "send", "evalue", "score")
    hit_dict = tsv_file_dict_parse(bls_file, fieldnames=outfmt6_title)

    # drop_short_hit
    def drop_short_aln(aln_len, query_len):
        aln_len_limit = max(min_len, query_len * aln_ratio)
        if aln_len >= aln_len_limit:
            return 1
        else:
            return 0

    candi_hit_id = [ID for ID in hit_dict if
                    drop_short_aln(abs(int(hit_dict[ID]["sstart"]) - int(hit_dict[ID]["send"])),
                                   len(record_dict[hit_dict[ID]["query"]].seqs))]

    output_list = []
    for ID in candi_hit_id:
        loc_range = (int(hit_dict[ID]["send"]), int(hit_dict[ID]["sstart"]))
        mirkey_start = min(loc_range)
        mirkey_end = max(loc_range)
        loc_delta = int(hit_dict[ID]["send"]) - int(hit_dict[ID]["sstart"])
        mirlen = abs(loc_delta) + 1

        if loc_delta > 0:
            strand = "+"
        else:
            strand = "-"

        loc_center = min(loc_range) + int(0.5 * mirlen)
        fold_start = max(1, loc_center - (int(0.5 * foldsize)))
        fold_stop = min(loc_center + (int(0.5 * foldsize)), len(record_db_dict[hit_dict[ID]["subject"]].seqs))

        fold_seq = sub_seq(record_db_dict[hit_dict[ID]["subject"]].seqs, fold_start, fold_stop, strand, True)

        cmd_string = "echo %s | %s --noPS " % (fold_seq, RNAfold_path)
        flag_tmp, output, error = lib.common.os.cmd_run(cmd_string, silence=True)
        output = output.decode()
        brax = output.split("\n")[1].split(" ")[0]

        check_mir_star_output = check_mir_star(brax, strand, fold_start, fold_stop, mirkey_start, mirlen)

        if len(check_mir_star_output) == 1:
            continue

        star_left, strand, mdz = check_mir_star_output

        query = hit_dict[ID]["query"]
        chr = hit_dict[ID]["subject"]
        mirkey_start = mirkey_start
        mirkey_end = mirkey_end
        strand = strand
        mir_seq = sub_seq(record_db_dict[hit_dict[ID]["subject"]].seqs, mirkey_start, mirkey_end, strand, True)
        mistar_start = star_left
        mistar_end = star_left + mdz - 1
        mistar_seq = sub_seq(record_db_dict[hit_dict[ID]["subject"]].seqs, mistar_start, mistar_end, strand, True)
        display_start = max(1, min(mistar_start, mistar_end, mirkey_start, mirkey_end) - 10)
        display_end = min(len(record_db_dict[hit_dict[ID]["subject"]].seqs),
                          max(mistar_start, mistar_end, mirkey_start, mirkey_end) + 10)
        display_fold_seq = sub_seq(record_db_dict[hit_dict[ID]["subject"]].seqs, display_start, display_end, strand,
                                   True)

        cmd_string = "echo %s | %s --noPS " % (display_fold_seq, RNAfold_path)
        flag_tmp, output, error = lib.common.os.cmd_run(cmd_string, silence=True)
        output = output.decode()
        display_fold_brax = output.split("\n")[1].split(" ")[0]

        output_list.append((query, chr, strand, mirkey_start, mirkey_end, mir_seq, mistar_start, mistar_end, mistar_seq,
                            display_start, display_end, display_fold_seq, display_fold_brax))
    return output_list


def SSRead_parse(input_file):
    with open(input_file, 'r') as f:
        for each_line in f:
            matchObj = re.match(
                r'^(Cluster_\d+) Original Location: (\S+):(\d+)-(\d+) Displayed Location: (\S+):(\d+)-(\d+) Strand: (\S).*',
                each_line)
            if matchObj:
                ID, Ori_Loc_chr, Ori_Loc_start, Ori_Loc_end, Dis_Loc_chr, Dis_Loc_start, Dis_Loc_end, strand = matchObj.groups()
                continue

            matchObj = re.match(r"^(\.*)([^\.]+)(\.*) miRNA l=(\d+) a=(\d+).*", each_line)
            if matchObj:
                miRNA_seq_l, miRNA_seq, miRNA_seq_r, miRNA_len, miRNA_aln = matchObj.groups()
                miRNA_seq = miRNA_seq.upper()
                left_len = len(miRNA_seq_l)
                right_len = len(miRNA_seq_r)
                if strand == "+":
                    miRNA_start = int(Dis_Loc_start) + int(left_len)
                    miRNA_end = int(Dis_Loc_end) - int(right_len)
                else:
                    miRNA_start = int(Dis_Loc_start) + int(right_len)
                    miRNA_end = int(Dis_Loc_end) - int(left_len)
                continue

            matchObj = re.match(r"^(\.*)([^\.]+)(\.*) miRNA-star l=(\d+) a=(\d+).*", each_line)
            if matchObj:
                miRNA_star_seq_l, miRNA_star_seq, miRNA_star_seq_r, miRNA_star_len, miRNA_star_aln = matchObj.groups()
                miRNA_star_seq = miRNA_star_seq.upper()
                left_len = len(miRNA_star_seq_l)
                right_len = len(miRNA_star_seq_r)
                if strand == "+":
                    miRNA_star_start = int(Dis_Loc_start) + int(left_len)
                    miRNA_star_end = int(Dis_Loc_end) - int(right_len)
                else:
                    miRNA_star_start = int(Dis_Loc_start) + int(right_len)
                    miRNA_star_end = int(Dis_Loc_end) - int(left_len)
                continue

    output_list = (
        input_file, ID, Ori_Loc_chr, Ori_Loc_start, Ori_Loc_end, Dis_Loc_chr, Dis_Loc_start, Dis_Loc_end, strand,
        miRNA_start, miRNA_end, miRNA_seq, miRNA_len, miRNA_aln, miRNA_star_start, miRNA_star_end, miRNA_star_seq,
        miRNA_star_len, miRNA_star_aln)

    return output_list


def bam_support(bam_file, contig, start, end, strand, sequence="", rpmm=False):
    """
    :bam_file = "/lustre/home/xuyuxing/Work/miRNA/Cau_two_assembly/ShortStack/Cau_map/de_novo_map/A3130.SS/A3130.filtered.10-42.bam"
    :contig = "C063N"
    :start = 26963
    :end = 26986
    :strand = "-"
    :sequence = 'CGCUUGUUAGAUCUUGCGUUCCUU'
    :return: 67
    """
    bam_file_object = pysam.AlignmentFile(bam_file, "rb")

    match_count = 0
    read_seq_raw = ""
    iter = bam_file_object.fetch(contig, start, end)
    all_read_seq = []
    for x in iter:
        if strand == "-" and x.is_reverse:
            read_seq_raw = x.get_forward_sequence()
        elif strand == "+" and not x.is_reverse:
            read_seq_raw = x.seq

        read_seq = re.sub('T', 'U', read_seq_raw)
        all_read_seq.append(read_seq)

        if not sequence == "":
            if read_seq == sequence:
                match_count = match_count + 1
    mapped_reads = bam_file_object.mapped
    bam_file_object.close()

    if not sequence == "":
        if not rpmm:
            return match_count
        else:
            rpmm_value = match_count / mapped_reads * 1000000
            return (match_count, rpmm_value)
    else:
        return Counter(all_read_seq)


def bam_support_block(bam_file, contig, start, end, strand, sequence):
    if not os.path.exists(bam_file + ".bai"):
        cmd_string = "samtools index %s" % bam_file
        flag_tmp, output, error = lib.common.os.cmd_run(cmd_string, silence=True)

    bam_file_object = pysam.AlignmentFile(bam_file, "rb")
    start = int(start)
    end = int(end)
    read_seq_raw = ""
    iter = bam_file_object.fetch(contig, start, end)
    blocks = []
    for x in iter:
        if strand == "-" and x.is_reverse:
            read_seq_raw = x.get_forward_sequence()
        elif strand == "+" and not x.is_reverse:
            read_seq_raw = x.seq
        read_seq = re.sub('T', 'U', read_seq_raw)
        if read_seq == sequence:
            blocks.append(tuple(x.blocks[0]))
    bam_file_object.close()

    blocks = Counter(blocks)
    blocks = sorted(blocks, key=lambda x: blocks[x], reverse=True)

    return (blocks[0][0] + 1, blocks[0][1])


def unique_miRNA_type(input_list, input_dir, contig, data_type):
    # choose miRNA type
    miRNA_type_list = {}
    for sample, cluster_id in input_list:
        miRNA_type = tuple([input_dir[sample][data_type][cluster_id][i] for i in (3, 4, 2, 5)])
        if not miRNA_type in miRNA_type_list: miRNA_type_list[miRNA_type] = []
        miRNA_type_list[miRNA_type].append((sample, cluster_id))

    miRNA_type_support = []
    for miRNA_type in miRNA_type_list:
        total_support = 0
        for sample, cluster_id in miRNA_type_list[miRNA_type]:
            bam_file = input_dir[sample]["bam"]
            start, end, strand, sequence = miRNA_type
            # print(bam_file, contig, start, end, strand, sequence)
            tmp_support = bam_support(bam_file, contig, start, end, strand, sequence)
            total_support = total_support + tmp_support
        if total_support == 0:
            continue
        miRNA_type_support.append((miRNA_type, total_support))

    miRNA_type_support = sorted(miRNA_type_support, key=lambda x: x[1], reverse=True)

    keep_miRNA_type = []
    for i in range(len(miRNA_type_support)):
        if i + 2 > len(miRNA_type_support):
            keep_miRNA_type.append(miRNA_type_support[i][0])
            break
        if miRNA_type_support[i][1] / miRNA_type_support[i + 1][1] > 2.0:
            keep_miRNA_type.append(miRNA_type_support[i][0])
            break
        else:
            keep_miRNA_type.append(miRNA_type_support[i][0])

    # choose miRNA star type
    all_output = []

    miRNA_star_type_dict = {}
    for sample, cluster_id in input_list:
        miRNA_type = tuple([input_dir[sample][data_type][cluster_id][i] for i in (3, 4, 2, 5)])
        if not miRNA_type in keep_miRNA_type: continue
        if not miRNA_type in miRNA_star_type_dict: miRNA_star_type_dict[miRNA_type] = {}
        miRNA_star_type = tuple([input_dir[sample][data_type][cluster_id][i] for i in (6, 7, 2, 8)])
        if not miRNA_star_type in miRNA_star_type_dict[miRNA_type]: miRNA_star_type_dict[miRNA_type][
            miRNA_star_type] = []
        miRNA_star_type_dict[miRNA_type][miRNA_star_type].append((sample, cluster_id))

    miRNA_star_type_support = {}
    for miRNA_type in miRNA_star_type_dict:
        miRNA_star_type_support[miRNA_type] = []
        for miRNA_star_type in miRNA_star_type_dict[miRNA_type]:
            total_support = 0
            for sample, cluster_id in miRNA_star_type_dict[miRNA_type][miRNA_star_type]:
                bam_file = input_dir[sample]["bam"]
                start, end, strand, sequence = miRNA_star_type
                # print(bam_file, contig, start, end, strand, sequence)
                tmp_support = bam_support(bam_file, contig, start, end, strand, sequence)
                total_support = total_support + tmp_support
            miRNA_star_type_support[miRNA_type].append((miRNA_star_type, total_support))

        miRNA_star_type_support[miRNA_type] = sorted(miRNA_star_type_support[miRNA_type], key=lambda x: x[1],
                                                     reverse=True)

        keep_miRNA_star_type = miRNA_star_type_support[miRNA_type][0][0]
        all_output.append(tuple(list(miRNA_type) + list(keep_miRNA_star_type)))

    return all_output


def block_to_miRNA_seq(contig, strand, start, end, input_dir, reference_genome_dict, support_read_rpmm=1,
                       orgin_seq=False):
    """
    :param contig="C017N"
    :param strand="+"
    :param start=4098941
    :param end=4098961
    :param input_dir:
    :return:
    """
    if not orgin_seq:
        read_sample_dict = {}
        for sample in input_dir:
            if sample == "homo_SS": continue
            candi = bam_support(input_dir[sample]["bam"], contig, start, end, strand)
            for i in candi:
                if candi[i] < 10:
                    continue
                if i not in read_sample_dict: read_sample_dict[i] = {}
                read_sample_dict[i][sample] = candi[i]

        sort_list_tmp = []
        for i in read_sample_dict:
            for sample in read_sample_dict[i]:
                sort_list_tmp.append((i, sample, read_sample_dict[i][sample]))

        if len(sort_list_tmp) == 0:
            return ""

        sort_list_tmp = sorted(sort_list_tmp, key=lambda x: x[2], reverse=True)

        best_miRNA_mature = sort_list_tmp[0][0]

        return best_miRNA_mature
    else:
        best_miRNA_mature = reference_genome_dict[contig].sub(start, end, strand, True)
        flag = 0
        for sample in input_dir:
            if sample == "homo_SS": continue
            rpmm = bam_support(input_dir[sample]["bam"], contig, start, end, strand, sequence=best_miRNA_mature,
                               rpmm=True)
            # print(rpmm)
            if rpmm >= support_read_rpmm:
                flag = 1
        if flag == 1:
            return best_miRNA_mature
        else:
            return ""


def miRNA_name(miRNA_seq, mirbase_fasta, temp_file):
    with open(temp_file, "w") as f:
        f.write(">%s\n%s\n" % ("temp", miRNA_seq))
    cmd_string = "%s -3 -m 8CD -n -Q -f -16 -r +15/0 -g -16 -w 25 -W 25 -E 10000 -U %s %s" % (
        ssearch_path, temp_file, mirbase_fasta)
    flag_tmp, output, error = lib.common.os.cmd_run(cmd_string, silence=True)
    output = output.decode()
    miRNA_name_dict = {}
    for each_line in output.split("\n"):
        if re.match(r'^#', each_line):
            continue
        info = each_line.split("\t")
        if len(info) == 1:
            continue
        name = info[1]
        aln_code = info[12]
        matches_num = 0
        for i in re.findall(r'(\d+)M', aln_code):
            matches_num = matches_num + int(i)
        if len(miRNA_seq) - matches_num > 2:
            continue
        else:
            # print(record, name)
            name_space = name.split("-")
            speci = name_space[0]
            miRNA_name = name_space[1]
            matchObj = re.match(r'^(miR\d+).*', miRNA_name)
            if matchObj:
                miRNA_name = re.findall(r'^(miR\d+).*', miRNA_name)[0]
                if miRNA_name not in miRNA_name_dict: miRNA_name_dict[miRNA_name] = []
                miRNA_name_dict[miRNA_name].append(speci)
                miRNA_name_dict[miRNA_name] = list(set(miRNA_name_dict[miRNA_name]))
    if len(miRNA_name_dict) == 0:
        best_name = "Unknown"
    else:
        best_name = sorted(miRNA_name_dict, key=lambda x: len(miRNA_name_dict[x]), reverse=True)[0]

    return best_name


if __name__ == '__main__':

    import argparse

    # argument parse
    # ---------------------------------------------------------
    parser = argparse.ArgumentParser(
        prog='miRNA',
    )

    subparsers = parser.add_subparsers(title='subcommands', dest="subcommand_name")

    # argparse for miRNAblast
    parser_a = subparsers.add_parser('miRNAblast',
                                     help='use mature miRNA to blast a genome and fold subject to find new homo miRNA\n',
                                     description='use mature miRNA to blast a genome and fold subject to find new homo miRNA')
    parser_a.add_argument("query_file", help="Path for query mature miRNA fasta", type=str)
    parser_a.add_argument("db_file", help="Path for genome assembly fasta", type=str)
    parser_a.add_argument("output_file", help="Path for output tab file", type=str)
    parser_a.add_argument("-r", "--aln_ratio",
                          help="the blast hit which length small than aln_ratio * query length will not be use. (default as 0.8)",
                          default=0.8, type=float)
    parser_a.add_argument("-l", "--min_len",
                          help="the blast hit which length small than min_len will not be use. (default as 18)",
                          default=18, type=int)
    parser_a.add_argument("-s", "--foldsize", help="sequence length which used in RNAfold (default as 300)",
                          default=300,
                          type=str)
    parser_a.add_argument("-p", "--num_threads", help="num of threads for blastn", default=1, type=int)

    # argparse for uniqMiRNA
    parser_a = subparsers.add_parser('uniqMiRNA',
                                     help='union blast miRNA subject to unique annotation\n',
                                     description='')

    parser_a.add_argument("input_file", help="output file from miRNAblast", type=str)
    parser_a.add_argument('output_file', type=str, help='unique miRNA annotation')

    # argparse for SSRead
    parser_a = subparsers.add_parser('SSRead',
                                     help='Read the ShortStack output in MIRNAs\n',
                                     description='')

    parser_a.add_argument("input_file", help="ShortStack output in MIRNAs", type=str)
    # parser_a.add_argument('output_file', type=str, help='unique miRNA annotation')

    # argparse for SSBamStat
    parser_a = subparsers.add_parser('SSBamStat',
                                     help='Read ShortStack bam output and stat the mapped and unmapped length distribution\n',
                                     description='')

    parser_a.add_argument("input_file", help="ShortStack bam output", type=str)
    parser_a.add_argument('output_file', type=str, help='output file')

    # argparse for SSResultsLen
    parser_a = subparsers.add_parser('SSResultsLen',
                                     help='ShortStack Results 20 21 22 23 24 在每个cluster中的比例与对应reads个数的分布\n',
                                     description='')

    parser_a.add_argument("input_file", help="ShortStack Results.txtt", type=str)
    parser_a.add_argument('output_file', type=str, help='output file')

    # argparse for SSResultsMerge
    parser_a = subparsers.add_parser('SSResultsMerge',
                                     help='Merge diff ShortStack Results to one annotation\n',
                                     description='')

    parser_a.add_argument("input_file_list",
                          help="a tab file include all ShortStack Results, first column is sample name and second column is ShortStack output dir.",
                          type=str)
    parser_a.add_argument('reference', type=str, help='reference genome file')
    parser_a.add_argument('mirbase_fasta', type=str, help='a fasta file included mature seq from mirbase')
    parser_a.add_argument('output_prefix', type=str, help='output prefix')
    parser_a.add_argument("-t", "--temp_dir",
                          help="temp dir which will be use for store some temp file. (default as ./)",
                          default="./", type=str)

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    # miRNAblast
    if args_dict["subcommand_name"] == "miRNAblast":
        query_file = args.query_file
        db_file = args.db_file
        output_file = args.output_file
        aln_ratio = args.aln_ratio
        min_len = args.min_len
        foldsize = args.foldsize
        num_threads = args.num_threads

        bls_file = os.path.dirname(os.path.abspath(output_file)) + "/" + str(os.getpid()) + ".tmp.bls"
        output = blast_and_hit_parse(query_file, db_file, bls_file, aln_ratio, min_len, foldsize, num_threads)

        with open(output_file, "w") as f:
            f.write(
                "query\tchr\tstrand\tmirna_start\tmirna_end\tmir_seq\tmistar_start\tmistar_end\tmistar_seq\tdisplay_start\tdisplay_end\tdisplay_fold_seq\tdisplay_fold_brax\n")
            for i in output:
                printer = ""
                for j in i:
                    printer = printer + str(j) + "\t"
                printer = printer.rstrip("\t")
                f.write(printer + "\n")

    # uniqMiRNA
    elif args_dict["subcommand_name"] == "uniqMiRNA":
        input_file = args.input_file
        output_file = args.output_file

        record_dict = tsv_file_dict_parse(input_file)

        ref_dict = {}
        for ID in record_dict:
            chr = record_dict[ID]['chr']
            strand = record_dict[ID]['strand']
            if not chr in ref_dict:
                ref_dict[chr] = {}
                ref_dict[chr]["+"] = {}
                ref_dict[chr]["+"]["ID"] = []
                ref_dict[chr]["-"] = {}
                ref_dict[chr]["-"]["ID"] = []
            ref_dict[chr][strand]["ID"].append(ID)

        num = 0
        group_dict = {}
        for chr in ref_dict:
            for strand in ref_dict[chr]:
                group_dict_tmp = group_by_intervals(
                    [(ID, int(record_dict[ID]["display_start"]), int(record_dict[ID]["display_end"])) for ID in
                     ref_dict[chr][strand]["ID"]], int=True)
                for range_key in group_dict_tmp:
                    group_dict["group_" + str(num)] = (chr, strand, range_key, tuple(group_dict_tmp[range_key]))
                    num = num + 1
        output_list = []
        for group_id in group_dict:
            chr, strand, range_key, record_list = group_dict[group_id]

            group_dict_tmp = group_by_intervals(
                [(ID, int(record_dict[ID]["mirna_start"]), int(record_dict[ID]["mirna_end"])) for ID in record_list],
                int=True)
            mirna_range = sorted(group_dict_tmp, key=lambda x: len(group_dict_tmp[x]), reverse=True)[0]

            group_dict_tmp = group_by_intervals(
                [(ID, int(record_dict[ID]["mistar_start"]), int(record_dict[ID]["mistar_end"])) for ID in record_list],
                int=True)
            mistar_range = sorted(group_dict_tmp, key=lambda x: len(group_dict_tmp[x]), reverse=True)[0]

            family_ID = []
            for ID in record_list:
                query_name = record_dict[ID]['query']
                for i in query_name.split("-"):
                    matchObj = re.match(r'(miR\d+).*', i)
                    if matchObj:
                        family_ID.append(matchObj.group(1))

            family_ID = Counter(family_ID)
            family_ID = sorted(family_ID, key=lambda x: family_ID[x], reverse=True)
            family_ID_best = family_ID[0]

            output_list.append((family_ID_best, chr, strand, mirna_range[0], mirna_range[1], mistar_range[0],
                                mistar_range[1], range_key[0], range_key[1]))

        with open(output_file, "w") as f:
            f.write(
                "family_name\tchr\tstrand\tmirna_start\tmirna_end\tmistar_start\tmistar_end\tdisplay_start\tdisplay_end\n")
            for i in output_list:
                printer = ""
                for j in i:
                    printer = printer + str(j) + "\t"
                printer = printer.rstrip("\t")
                f.write(printer + "\n")

    # SSRead
    elif args_dict["subcommand_name"] == "SSRead":
        input_file = args.input_file
        # output_file = args.output_file

        output_list = SSRead_parse(input_file)

        printer = ""
        for j in output_list:
            printer = printer + str(j) + "\t"
        printer = printer.rstrip("\t")
        print(printer)

    # SSBamStat
    elif args_dict["subcommand_name"] == "SSBamStat":
        from collections import Counter

        input_file = args.input_file
        output_file = args.output_file

        # input_file = "/lustre/home/xuyuxing/Work/miRNA/Cau/bam/Cau/test.bam"

        map_type_dict = {}
        min_len = float("inf")
        max_len = float("-inf")
        for each_line in bf.cmd_open(samtools_path + " view " + input_file):
            each_line = re.sub(r'\n', '', each_line)
            if each_line == "": continue
            info = each_line.split("\t")
            seqname = info[0]
            seqs_len = len(info[9])
            if seqs_len < min_len: min_len = seqs_len
            if seqs_len > max_len: max_len = seqs_len
            contig = info[2]
            if contig == "*":
                if not "unmapped" in map_type_dict: map_type_dict["unmapped"] = []
                map_type_dict["unmapped"].append(seqs_len)
                continue
            XY_Z = info[17]
            if not XY_Z in map_type_dict: map_type_dict[XY_Z] = []
            map_type_dict[XY_Z].append(seqs_len)

        map_type_dict_key = list(map_type_dict.keys())
        for XY_Z in map_type_dict_key:
            map_type_dict[XY_Z] = Counter(map_type_dict[XY_Z])

        with open(output_file, "w") as f:
            print_head = "\t"
            for XY_Z in map_type_dict_key:
                print_head = print_head + XY_Z + "\t"
            print_head = print_head.rstrip("\t")
            f.write(print_head + "\n")

            for i in range(min_len, max_len + 1):
                printer = "%d\t" % i
                for XY_Z in map_type_dict_key:
                    printer = printer + str(map_type_dict[XY_Z][i]) + "\t"
                printer = printer.rstrip("\t")
                f.write(printer + "\n")


    # SSResultsLen
    elif args_dict["subcommand_name"] == "SSResultsLen":
        from collections import Counter

        input_file = args.input_file
        output_file = args.output_file

        dict_load = tsv_file_dict_parse(input_file)

        reads_len = ["20", "21", "22", "23", "24"]

        reads_ratio_list = {}
        for len_tmp in reads_len:
            reads_ratio_list[len_tmp] = []
            for ID in dict_load:
                all_counts = 0
                for i in reads_len:
                    all_counts = all_counts + int(dict_load[ID][i])
                if all_counts == 0: continue
                ratio = (int(dict_load[ID][len_tmp]) / all_counts) * 100
                reads_ratio_list[len_tmp].append((ratio, int(dict_load[ID][len_tmp])))

        output_dict = {}
        for len_tmp in reads_len:
            output_dict[len_tmp] = {}
            for i in range(0, 100):
                output_dict[len_tmp][i] = 0
                for ratio, reads in reads_ratio_list[len_tmp]:
                    if ratio >= i and ratio < i + 1:
                        output_dict[len_tmp][i] = output_dict[len_tmp][i] + reads

    # SSResultsMerge
    elif args_dict["subcommand_name"] == "SSResultsMerge":
        input_file_list = args.input_file_list
        reference_file = args.reference
        output_prefix = args.output_prefix
        mirbase_fasta = args.mirbase_fasta
        temp_dir = args.temp_dir

        # Read all information from SS

        input_dir = tsv_file_dict_parse(input_file_list, seq=",", fieldnames=("key", "dir"), key_col="key",
                                        fields=["dir"])

        reference_genome_dict = {}
        for record in read_fasta_big(reference_file):
            reference_genome_dict[record.seqname] = record

        for sample in input_dir:
            dir_tmp = input_dir[sample]["dir"]
            for file in os.listdir(dir_tmp):
                if re.match(r'.*\.bam$', file):
                    input_dir[sample]["bam"] = os.path.join(dir_tmp, file)
                if file == "Results.txt":
                    file_path = os.path.join(dir_tmp, file)
                    input_dir[sample]["results"] = tsv_file_dict_parse(file_path, key_col="Name")

            input_dir[sample]["de_novo"] = {}
            for file in os.listdir(dir_tmp + "/MIRNAs"):
                file_path = os.path.join(dir_tmp + "/MIRNAs", file)
                output_list = SSRead_parse(file_path)
                input_dir[sample]["de_novo"][output_list[1]] = tuple(
                    [output_list[i] for i in (1, 2, 8, 9, 10, 11, 14, 15, 16)])

            input_dir[sample]["N15"] = {}
            for cluster_id in input_dir[sample]["results"]:
                if input_dir[sample]["results"][cluster_id]['MIRNA'] == 'N15':
                    strand = input_dir[sample]["results"][cluster_id]['Strand']
                    if strand == ".":
                        # print("Strand Error", sample, cluster_id)
                        continue
                    DicerCall = int(input_dir[sample]["results"][cluster_id]['DicerCall'])
                    if DicerCall < 20 or DicerCall > 24:
                        # print("DicerCall Error", sample, cluster_id)
                        continue

                    contig, raw_start, raw_end = re.findall(r"(\S+):(\d+)-(\d+)",
                                                            input_dir[sample]["results"][cluster_id]['#Locus'])[0]
                    MajorRNA = input_dir[sample]["results"][cluster_id]['MajorRNA']

                    block_accurate = bam_support_block(input_dir[sample]["bam"], contig, raw_start, raw_end, strand,
                                                       MajorRNA)

                    refseq = reference_genome_dict[contig].seqs

                    output_mistar = mistar_get(block_accurate[0], block_accurate[1] - block_accurate[0] + 1, refseq,
                                               strand, raw_start, raw_end, 300, mir_seq=MajorRNA)

                    if len(output_mistar) == 1:
                        # print(output_mistar, sample, cluster_id)
                        continue
                    else:
                        (mirkey_start, mirkey_end, mir_seq, mistar_start, mistar_end, mistar_seq) = output_mistar

                    if not (mirkey_start, mirkey_end) == block_accurate:
                        # print("Error2", sample, cluster_id)
                        continue
                    else:
                        input_dir[sample]["N15"][cluster_id] = (
                            cluster_id, contig, strand, mirkey_start, mirkey_end, mir_seq, mistar_start, mistar_end,
                            mistar_seq)

        # Load all sample de novo miRNA and merge them

        de_novo_dict = {}
        for sample in input_dir:
            for cluster_id in input_dir[sample]["de_novo"]:
                record_tuple = input_dir[sample]["de_novo"][cluster_id]
                if not record_tuple[1] in de_novo_dict: de_novo_dict[record_tuple[1]] = {"+": {}, "-": {}}
                index_miRNA = (sample, record_tuple[0])
                de_novo_dict[record_tuple[1]][record_tuple[2]][index_miRNA] = record_tuple

        good_miRNA = {}
        for contig in de_novo_dict:
            if contig not in good_miRNA: good_miRNA[contig] = {"+": [], "-": []}
            for strand in de_novo_dict[contig]:
                interval_finder_input = []
                for index_miRNA in de_novo_dict[contig][strand]:
                    interval_finder_input.append((index_miRNA, de_novo_dict[contig][strand][index_miRNA][3],
                                                  de_novo_dict[contig][strand][index_miRNA][4]))
                grouped_intervals = group_by_intervals(interval_finder_input, int=True)

                for group in grouped_intervals:
                    input_tmp = grouped_intervals[group]
                    uniq_miRNA = unique_miRNA_type(input_tmp, input_dir, contig, "de_novo")
                    for tmp_info in uniq_miRNA:
                        miRNA_info = [contig, strand] + [tmp_info[i] for i in (0, 1, 3, 4, 5, 7)]
                        good_miRNA[contig][strand].append(miRNA_info)

        # Load all sample N15 miRNA and del de novo one and merge them

        N15_dict = {}
        for sample in input_dir:
            if sample == "homo_SS":
                continue
            for cluster_id in input_dir[sample]["N15"]:
                record_tuple = input_dir[sample]["N15"][cluster_id]
                if not record_tuple[1] in N15_dict: N15_dict[record_tuple[1]] = {"+": {}, "-": {}}
                index_miRNA = (sample, record_tuple[0])
                N15_dict[record_tuple[1]][record_tuple[2]][index_miRNA] = record_tuple

        good_N15 = {}
        for contig in N15_dict:
            if contig not in good_N15: good_N15[contig] = {"+": [], "-": []}
            for strand in N15_dict[contig]:
                interval_finder_input = []
                for index_miRNA in N15_dict[contig][strand]:
                    interval_finder_input.append((index_miRNA, N15_dict[contig][strand][index_miRNA][3],
                                                  N15_dict[contig][strand][index_miRNA][4]))
                grouped_intervals = group_by_intervals(interval_finder_input, int=True)

                for group in grouped_intervals:
                    input_tmp = grouped_intervals[group]
                    uniq_miRNA = unique_miRNA_type(input_tmp, input_dir, contig, "N15")
                    for tmp_info in uniq_miRNA:
                        miRNA_info = [contig, strand] + [tmp_info[i] for i in (0, 1, 3, 4, 5, 7)]
                        de_novo_flag = 0
                        if contig in good_miRNA:
                            for good_miRNA_info in good_miRNA[contig][strand]:
                                a, b = section((good_miRNA_info[2], good_miRNA_info[3]), (miRNA_info[2], miRNA_info[3]))
                                if a:
                                    # print("Founded", miRNA_info)
                                    de_novo_flag = 1
                        if de_novo_flag == 0:
                            good_N15[contig][strand].append(miRNA_info)

        # Name miRNA

        temp_file = temp_dir + "/temp.fa"
        for contig in good_miRNA:
            for strand in good_miRNA[contig]:
                named_list = []
                for record in good_miRNA[contig][strand]:
                    miRNA_seq = record[4]
                    best_name = miRNA_name(miRNA_seq, mirbase_fasta, temp_file)
                    record.append(best_name)
                    # print(record)
                    named_list.append(record)
                good_miRNA[contig][strand] = named_list

        for contig in good_N15:
            for strand in good_N15[contig]:
                named_list = []
                for record in good_N15[contig][strand]:
                    miRNA_seq = record[4]
                    best_name = miRNA_name(miRNA_seq, mirbase_fasta, temp_file)
                    if best_name == "Unknown":
                        continue
                    record.append(best_name)
                    # print(record)
                    named_list.append(record)
                good_N15[contig][strand] = named_list

        # Count and rpmm

        sample_list = input_dir.keys()
        count_rpmm_dict = {}
        for contig in good_miRNA:
            for strand in good_miRNA[contig]:
                for record in good_miRNA[contig][strand]:
                    record.append("de_novo")
                    record = tuple(record)
                    count_rpmm_dict[record] = {}
                    start = record[2]
                    end = record[3]
                    seqs = record[4]
                    for i in sample_list:
                        bam_file = input_dir[i]['bam']
                        count, rpmm = bam_support(bam_file, contig, start, end, strand, seqs, True)
                        count_rpmm_dict[record][i] = (count, rpmm)

        for contig in good_N15:
            for strand in good_N15[contig]:
                for record in good_N15[contig][strand]:
                    record.append("N15")
                    record = tuple(record)
                    count_rpmm_dict[record] = {}
                    start = record[2]
                    end = record[3]
                    seqs = record[4]
                    for i in sample_list:
                        bam_file = input_dir[i]['bam']
                        count, rpmm = bam_support(bam_file, contig, start, end, strand, seqs, True)
                        count_rpmm_dict[record][i] = (count, rpmm)

        # output
        counts_file = output_prefix + ".counts.tsv"
        with open(counts_file, "w") as f:
            header = "contig\tstrand\tmiRNA_start\tmiRNA_end\tmiRNA_seq\tmistar_start\tmistar_end\tmistar_seq\tmiRNA_family\tsource\t"
            for i in sample_list:
                header = header + i + "\t"
            header = header.rstrip("\t")
            f.write(header + "\n")

            for record in count_rpmm_dict:
                printer = ""
                for i in record:
                    printer = printer + str(i) + "\t"

                for i in sample_list:
                    printer = printer + str(count_rpmm_dict[record][i][0]) + "\t"
                printer.rstrip("\t")

                f.write(printer + "\n")

        rpmm_file = output_prefix + ".rpmm.tsv"
        with open(rpmm_file, "w") as f:
            header = "contig\tstrand\tmiRNA_start\tmiRNA_end\tmiRNA_seq\tmistar_start\tmistar_end\tmistar_seq\tmiRNA_family\tsource\t"
            for i in sample_list:
                header = header + i + "\t"
            header = header.rstrip("\t")
            f.write(header + "\n")

            for record in count_rpmm_dict:
                printer = ""
                for i in record:
                    printer = printer + str(i) + "\t"

                for i in sample_list:
                    printer = printer + str(count_rpmm_dict[record][i][1]) + "\t"
                printer.rstrip("\t")

                f.write(printer + "\n")

    # test
    # 24nt_source
    elif args_dict["subcommand_name"] == "24nt_source":
        # Todo
        input_file_list = args.input_file_list
        reference_file = args.reference
        output_prefix = args.output_prefix
        mirbase_fasta = args.mirbase_fasta
        temp_dir = args.temp_dir

        # Read all information from SS

        """
        input_file_list = "/lustre/home/xuyuxing/Work/miRNA/Cau_two_assembly/ShortStack/Cau_map/merge/SS.list"
        """

        input_dir = tsv_file_dict_parse(input_file_list, seq=",", fieldnames=("key", "dir"), key_col="key",
                                        fields=["dir"])

        for sample in input_dir:
            dir_tmp = input_dir[sample]["dir"]
            for file in os.listdir(dir_tmp):
                if re.match(r'.*\.bam$', file):
                    input_dir[sample]["bam"] = os.path.join(dir_tmp, file)
                if file == "Results.txt":
                    file_path = os.path.join(dir_tmp, file)
                    input_dir[sample]["results"] = file_path


        def deep_analy(file_path, relate_threshold_reads_num=3, relate_threshold_complexity=0.98):
            import numpy as np
            """
            file_path = "/lustre/home/xuyuxing/Work/miRNA/Cau_two_assembly/ShortStack/Cau_map/de_novo_map/A2427.SS/Results.txt"
            relate_threshold_reads_num = 10
            relate_threshold_complexity= 0.6
            """
            reads_len = ["20", "21", "22", "23", "24"]
            result_dict = tsv_file_dict_parse(file_path, key_col="Name")
            total_sum_dict = {"DicerCall_reads_sum": {"20": 0, "21": 0, "22": 0, "23": 0, "24": 0}}

            for cluster_id in result_dict:
                # print(cluster_id)
                cluster_tmp = result_dict[cluster_id]

                DicerCall_reads = 0
                for i in reads_len:
                    DicerCall_reads = DicerCall_reads + int(cluster_tmp[i])
                    total_sum_dict["DicerCall_reads_sum"][i] = total_sum_dict["DicerCall_reads_sum"][i] + int(
                        cluster_tmp[i])
                result_dict[cluster_id]["DicerCall_reads"] = DicerCall_reads

                for i in reads_len:
                    if DicerCall_reads == 0:
                        ratio = 0.0
                    else:
                        ratio = float(cluster_tmp[i]) / float(DicerCall_reads)
                    result_dict[cluster_id][i + "_ratio"] = ratio

            total_sum_dict["DicerCall_length_ratio_his"] = {}
            for i in reads_len:
                ratio_list = [result_dict[cluster_id][i + "_ratio"] for cluster_id in result_dict]
                total_sum_dict["DicerCall_length_ratio_his"][i] = np.histogram(ratio_list, bins=np.arange(101) / 100)

            total_sum_dict["DicerCall_length_related_cluster"] = {}
            for i in reads_len:
                filterd_len_cluster = []
                for cluster_id in result_dict:
                    cluster_tmp = result_dict[cluster_id]
                    if cluster_tmp["DicerCall"] == i and int(cluster_tmp[i]) >= relate_threshold_reads_num and float(
                            cluster_tmp["Complexity"]) <= relate_threshold_complexity:
                        filterd_len_cluster.append(cluster_id)

                filterd_len_cluster_reads = 0
                for cluster_id in filterd_len_cluster:
                    filterd_len_cluster_reads = filterd_len_cluster_reads + int(result_dict[cluster_id][i])

                print(i, filterd_len_cluster_reads, total_sum_dict["DicerCall_reads_sum"][i],
                      filterd_len_cluster_reads / total_sum_dict["DicerCall_reads_sum"][i])

                total_sum_dict["DicerCall_length_related_cluster"][i] = filterd_len_cluster


        for i in [3, 5, 10, 15]:
            for j in [0.6, 0.8, 0.9, 0.95]:
                print(i, j)
                # deep_analy("/lustre/home/xuyuxing/Work/miRNA/Cau_two_assembly/ShortStack/Cau_map/de_novo_map/A2427.SS/Results.txt",i,j)
                deep_analy(
                    "/lustre/home/xuyuxing/Work/miRNA/Cau_pacbio_genome/ShortStack/2018/test_Ath/TEST_DATA/test1/Results.txt",
                    i, j)

        with open("temp.txt", "w") as f:
            for i in total_sum_dict["DicerCall_length_related_cluster"]["24"]:
                f.write(i + "\n")



from api.xuyuxing.evolution.badirate import main_pipeline, F_index, get_branch_name
from lib.common.fileIO import tsv_file_dict_parse
import numpy as np
from config import badirate_path
from collections import OrderedDict
from lib.common.util import printer_list
from lib.common.os import mkdir, rmdir, multiprocess_running
from io import StringIO
from Bio import Phylo
from lib.common.evolution.tree_operate import get_sons
import re


def badirate_one(c_id, gene_count_tuple, species_name_list, species_tree, work_dir, just_load):
    # build size file
    tmp_dir = work_dir + "/" + c_id
    mkdir(tmp_dir, True)

    try:
        size_file = tmp_dir+"/size_file"
        with open(size_file, 'w') as f:
            head_string = printer_list(
                species_name_list, sep='\t', head="FAM_ID\t")
            f.write(head_string+"\n")
            count_string = printer_list(
                gene_count_tuple, sep='\t', head=c_id+"\t")
            f.write(count_string+"\n")

        output = main_pipeline(c_id, size_file, species_tree,
                               tmp_dir+"/run", badirate_path, None, True, just_load)

        return output

    except:
        return None


def get_leaf_branch(labeled_tree_string):
    labeled_tree = StringIO(labeled_tree_string)

    tree = Phylo.read(labeled_tree, 'newick')

    all_branch = []
    species_name_map = {}
    for clade in tree.find_clades(order='level'):
        if clade.is_terminal():
            continue
        else:
            for son in get_sons(clade):
                clade_name = clade.confidence
                if son.is_terminal():
                    son_speci_name, son_name = re.match(
                        r'^(\S+)_(\d+)$', son.name).groups()
                    species_name_map[son_speci_name] = son_name
                else:
                    son_name = son.confidence

                all_branch.append("%s->%s" % (clade_name, son_name))

    for species_name in species_name_map:
        for i in all_branch:
            if i.split(">")[-1] == species_name_map[species_name]:
                species_name_map[species_name] = (
                    species_name_map[species_name], i)
                break

    return species_name_map


def gene_family_stat_main(Orthogroups_GeneCount_tsv, species_tree, species_name_list, conserved_species_name_list, work_dir, threads, log_file, output_prefix, just_load=False):
    mkdir(work_dir, True)

    # get gene count
    gene_count_dict = tsv_file_dict_parse(
        Orthogroups_GeneCount_tsv, key_col='Orthogroup', fields=species_name_list)
    for OG_id in gene_count_dict:
        for i in gene_count_dict[OG_id]:
            gene_count_dict[OG_id][i] = int(gene_count_dict[OG_id][i])

    # # test
    # import random
    # gene_count_dict = {i: gene_count_dict[i]
    #                    for i in random.sample(list(gene_count_dict), 56)}
    # # test end

    # get f index
    F_index_dict = OrderedDict()
    for OG_id in gene_count_dict:
        gene_count_tuple = tuple([gene_count_dict[OG_id][i]
                                  for i in species_name_list])
        f_index_tuple = F_index(gene_count_tuple)
        F_index_dict[OG_id] = OrderedDict()
        for i in range(len(species_name_list)):
            F_index_dict[OG_id][species_name_list[i]] = f_index_tuple[i]

    # get badirate
    badirate_input_dict_tmp = {}
    single_species_OG_list = []
    huge_OG_list = []
    for OG_id in gene_count_dict:

        f_index_tuple = tuple([F_index_dict[OG_id][i]
                               for i in F_index_dict[OG_id]])
        gene_count_tuple = tuple([gene_count_dict[OG_id][i]
                                  for i in gene_count_dict[OG_id]])

        if 1.0 in f_index_tuple:
            single_species_OG_list.append(OG_id)
        elif len([i for i in gene_count_tuple if i >= 100]) > 0:
            huge_OG_list.append(OG_id)
        else:
            if gene_count_tuple not in badirate_input_dict_tmp:
                badirate_input_dict_tmp[gene_count_tuple] = []

            badirate_input_dict_tmp[gene_count_tuple].append(OG_id)

    badirate_input_dict = {}
    num = 0
    for gene_count_tuple in badirate_input_dict_tmp:
        badirate_input_dict["C"+str(num)] = {
            "tuple": gene_count_tuple,
            "OG": badirate_input_dict_tmp[gene_count_tuple]
        }
        num += 1

    # run badirate

    args_list = []
    args_id_list = []
    for c_id in badirate_input_dict:
        gene_count_tuple = badirate_input_dict[c_id]["tuple"]
        args_list.append(
            (c_id, gene_count_tuple, species_name_list, species_tree, work_dir, just_load))
        args_id_list.append(c_id)

        # badirate_one(c_id, gene_count_tuple, species_name_list, species_tree, work_dir)

    # mp_out = multiprocess_running(badirate_one, args_list, threads,
    #                               log_file=log_file, silence=True, args_id_list=args_id_list, timeout=86400)

    # mp_out = multiprocess_running(badirate_one, args_list, threads,
    #                               log_file=log_file, silence=True, args_id_list=args_id_list, timeout=3600)

    mp_out = multiprocess_running(badirate_one, args_list, threads,
                                  log_file=log_file, silence=True, args_id_list=args_id_list)

    for c_id in mp_out:
        output = mp_out[c_id]['output']
        error = mp_out[c_id]['error']

        if error == "timeout":
            badirate_input_dict[c_id]["output"] = None
        else:
            if output:
                badirate_input_dict[c_id]["output"] = output
            else:
                badirate_input_dict[c_id]["output"] = None

    # make report
    report_dict = {}
    for c_id in badirate_input_dict:
        gene_count_tuple = badirate_input_dict[c_id]['tuple']
        output = badirate_input_dict[c_id]['output']
        OG_list = badirate_input_dict[c_id]['OG']

        species_num = len(gene_count_tuple) - gene_count_tuple.count(0)
        gene_sum = sum(gene_count_tuple)

        if [gene_count_tuple[species_name_list.index(i)] for i in conserved_species_name_list].count(0) == 0:
            conserved = True
        else:
            conserved = False

        for OG_id in OG_list:

            report_dict[OG_id] = {
                'gene_count': gene_count_tuple,
                'F_index':  tuple([F_index_dict[OG_id][i] for i in species_name_list]),
                'species_num': species_num,
                'gene_sum': gene_sum,
                'conserved': conserved,
            }

            if output is None:
                report_dict[OG_id]['Gain'] = []
                report_dict[OG_id]['Loss'] = []
                report_dict[OG_id]['Expansion'] = []
                report_dict[OG_id]['Contraction'] = []
                report_dict[OG_id]['Likeilhood'] = -999
            else:
                report_dict[OG_id]['Gain'] = output[1]
                report_dict[OG_id]['Loss'] = output[2]
                report_dict[OG_id]['Expansion'] = output[3]
                report_dict[OG_id]['Contraction'] = output[4]
                report_dict[OG_id]['Likeilhood'] = output[5]
                labeled_tree_string = output[6]

    species_name_map = get_leaf_branch(labeled_tree_string)

    for OG_id in gene_count_dict:
        if OG_id in report_dict:
            continue

        gene_count_tuple = tuple([gene_count_dict[OG_id][i]
                                  for i in species_name_list])
        f_index_tuple = tuple([F_index_dict[OG_id][i]
                               for i in species_name_list])

        species_num = len(gene_count_tuple) - gene_count_tuple.count(0)
        gene_sum = sum(gene_count_tuple)

        conserved = False

        Gains = []
        for i in gene_count_dict[OG_id]:
            if gene_count_dict[OG_id][i] != 0:
                Gains = [species_name_map[i][1]]

        report_dict[OG_id] = {
            'gene_count': gene_count_tuple,
            'F_index':  f_index_tuple,
            'species_num': species_num,
            'gene_sum': gene_sum,
            'conserved': conserved,
            'Gain': Gains,
            'Loss': [],
            'Expansion': [],
            'Contraction': [],
            'Likeilhood': -999
        }

    output_tsv_file = output_prefix + ".tsv"
    output_label_tre = output_prefix + ".label.tree"

    with open(output_label_tre, 'w') as f:
        f.write(labeled_tree_string)

    with open(output_tsv_file, 'w') as f:
        header_string = printer_list(["Family ID"] + species_name_list + ['Conserved',
                                                                          'Sum', 'Species', 'Gain', 'Loss', 'Expansion', 'Contraction', 'Likeilhood'] + species_name_list, sep='\t')
        f.write(header_string+"\n")

        for OG_id in gene_count_dict:
            Gains_string = printer_list(report_dict[OG_id]['Gain'], sep=',')
            Losses_string = printer_list(report_dict[OG_id]['Loss'], sep=',')
            Expansion_string = printer_list(
                report_dict[OG_id]['Expansion'], sep=',')
            Contraction_string = printer_list(
                report_dict[OG_id]['Contraction'], sep=',')

            output_list = [OG_id] + list(report_dict[OG_id]['gene_count']) + [report_dict[OG_id]['conserved'], report_dict[OG_id]['gene_sum'],
                                                                              report_dict[OG_id]['species_num'], Gains_string, Losses_string, Expansion_string, Contraction_string, report_dict[OG_id]['Likeilhood']] + list(report_dict[OG_id]['F_index'])
            output_string = printer_list(output_list, sep='\t')
            f.write(output_string+"\n")


if __name__ == "__main__":

    Orthogroups_GeneCount_tsv = '/lustre/home/xuyuxing/Work/Gel/gene_family_expand/pt_file/badirate/Orthogroups.GeneCount.tsv'
    species_tree = '/lustre/home/xuyuxing/Work/Gel/gene_family_expand/pt_file/badirate/species_tree.tre'
    species_name_list = ["Osa", "Dal", "Tze",
                         "Xvi", "Aof", "Ash", "Dca", "Peq", "Gel"]
    conserved_species_name_list = ["Osa", "Dal", "Tze", "Xvi", "Aof"]
    work_dir = '/lustre/home/xuyuxing/Work/Gel/gene_family_expand/pt_file/badirate/tmp'
    threads = 56
    log_file = '/lustre/home/xuyuxing/Work/Gel/gene_family_expand/pt_file/badirate/log'
    output_prefix = '/lustre/home/xuyuxing/Work/Gel/gene_family_expand/pt_file/badirate/genefamily'

    gene_family_stat_main(Orthogroups_GeneCount_tsv, species_tree, species_name_list,
                          conserved_species_name_list, work_dir, threads, log_file, output_prefix, just_load)

    #
    Orthogroups_GeneCount_tsv = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.6.hcluster/hcluster/hcluster.GeneCount.tsv"
    species_tree = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.6.hcluster/hcluster/SpeciesTree_rooted.txt"
    species_name_list = ["Atr", "Cka", "Lch", "Cnu", "Osa", "Zma",
                         "Acom", "Aof", "Gel", "Dca", "Peq", "Vpl", "Ash", "Xvi", "Dal"]
    conserved_species_name_list = [
        "Atr", "Cka", "Lch", "Cnu", "Osa", "Zma", "Acom", "Aof", "Xvi", "Dal"]
    work_dir = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.6.hcluster/hcluster/tmp"
    threads = 160
    log_file = work_dir+"/log"
    output_prefix = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.6.hcluster/hcluster/badirates"
    just_load = False

    gene_family_stat_main(Orthogroups_GeneCount_tsv, species_tree, species_name_list,
                          conserved_species_name_list, work_dir, threads, log_file, output_prefix, just_load=False)

    #
    Orthogroups_GeneCount_tsv = "/lustre/home/xuyuxing/Work/Orobanchaceae/publish/3.gene_family/2.expansion_contract/merge_OGs_num.tsv"
    species_tree = "/lustre/home/xuyuxing/Work/Orobanchaceae/publish/3.gene_family/2.expansion_contract/species_time.tre"
    species_name_list = ["Ath","Cau","Cca","Ini","Llu","Mgu","Ocu","Pae","Pcr","Pja","Sas","Sin","Sly"]
    conserved_species_name_list = ["Atr", "Cca", "Ini", "Llu", "Mgu", "Sly"]
    work_dir = "/lustre/home/xuyuxing/Work/Orobanchaceae/publish/3.gene_family/2.expansion_contract/tmp"
    threads = 160
    log_file = work_dir+"/log"
    output_prefix = "/lustre/home/xuyuxing/Work/Orobanchaceae/publish/3.gene_family/2.expansion_contract/badirates"
    just_load = False

    gene_family_stat_main(Orthogroups_GeneCount_tsv, species_tree, species_name_list,
                          conserved_species_name_list, work_dir, threads, log_file, output_prefix, just_load=False)



import re
import uuid
import shutil
from config import genewise_path
from lib.common.os import mkdir, cmd_run
from config import tmp_work_dir, genblasta_path
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation
from BCBio import GFF
from collections import OrderedDict


def genewise_run(query_file, subject_file, genewise_out, strand, pseudo=False, q_start=None, q_end=None):
    """
    /lustre/home/xuyuxing/Program/wise/wise2.4.1/src/bin/genewise -pretty -tfor -genes -gff -pseudo -pep -nosplice_gtag -init global -s 142 -t 263 q.fa T267555N0C000_4492499_4493404.fa
    """
    if pseudo:
        if strand == "+":
            cmd_string = genewise_path + " -pretty -tfor -genes -gff -pseudo -pep -nosplice_gtag -init global -s %d -t %d %s %s > %s" % (q_start, q_end, query_file, subject_file, genewise_out)
        else:
            cmd_string = genewise_path + " -pretty -trev -genes -gff -pseudo -pep -nosplice_gtag -init global -s %d -t %d %s %s > %s" % (q_start, q_end, query_file, subject_file, genewise_out)
    else:
        if strand == "+":
            cmd_string = genewise_path + " -pretty -tfor -genes -gff -pseudo -pep %s %s > %s" % (
                query_file, subject_file, genewise_out)
        else:
            cmd_string = genewise_path + " -pretty -trev -genes -gff -pseudo -pep %s %s > %s" % (
                query_file, subject_file, genewise_out)

    cmd_run(cmd_string, silence=True)

    return query_file, subject_file, genewise_out


def parse_align_sub(align_sub):
    p0 = re.compile('^$')
    p00 = re.compile('^\s+$')
    p_1 = re.compile('(...................\d) (.*)$')
    p_23 = re.compile('(                    ) (.*)$')
    p_wise_mark = re.compile('^.*.[^\d](\d+)')
    True_block = []
    num = 0
    flag1 = 0;
    flag2 = 0;
    flag3 = 0;
    flag4 = 0;
    flag5 = 0;
    flag6 = 0
    line1 = [];
    line2 = [];
    line3 = [];
    line4 = [];
    line5 = [];
    line6 = []
    for block in align_sub:
        block = re.sub(p0, '', block)
        block = re.sub(p00, '', block)
        lines = block.split('\n')
        block_start = 0
        for line in lines:
            if re.match('Score (.*) bits over entire alignment', line):
                genewise_score = float(re.findall('Score (.*) bits over entire alignment', line)[0])
            elif flag1 == 0:
                match = p_1.match(line)
                if match:
                    query_name = re.findall('^(\S+)\s+.*', line)[0]
                    flag1 = 1
                    wise_mark = match.group(1)
                    wise_str = match.group(2)
                    match2 = p_wise_mark.match(wise_mark)
                    query_block_start = match2.group(1)
                    line1 = wise_str
                    continue
            elif flag1 == 1 and flag2 == 0:
                match = p_23.match(line)
                if match:
                    flag2 = 1
                    wise_str = match.group(2)
                    line2 = wise_str
                    continue
            elif flag1 == 1 and flag2 == 1 and flag3 == 0:
                match = p_23.match(line)
                if match:
                    flag3 = 1
                    wise_str = match.group(2)
                    line3 = wise_str
                    continue
            elif flag1 == 1 and flag2 == 1 and flag3 == 1 and flag4 == 0:
                match = p_1.match(line)
                if match:
                    subject_name = re.findall('^(\S+)\s+.*', line)[0]
                    flag4 = 1
                    wise_mark = match.group(1)
                    wise_str = match.group(2)
                    match2 = p_wise_mark.match(wise_mark)
                    block_start = match2.group(1)
                    line4 = wise_str
                    continue
            elif flag1 == 1 and flag2 == 1 and flag3 == 1 and flag4 == 1 and flag5 == 0:
                match = p_23.match(line)
                if match:
                    flag5 = 1
                    wise_str = match.group(2)
                    line5 = wise_str
                    continue
            elif flag1 == 1 and flag2 == 1 and flag3 == 1 and flag4 == 1 and flag5 == 1 and flag6 == 0:
                match = p_23.match(line)
                if match:
                    flag6 = 1
                    wise_str = match.group(2)
                    line6 = wise_str
                    num = num + 1
                    True_block.append([num, block_start, query_block_start, [line1, line2, line3, line4, line5, line6]])
                    flag1 = 0;
                    flag2 = 0;
                    flag3 = 0;
                    flag4 = 0;
                    flag5 = 0;
                    flag6 = 0
                    line1 = [];
                    line2 = [];
                    line3 = [];
                    line4 = [];
                    line5 = [];
                    line6 = []
                    continue
    return True_block, query_name, subject_name, genewise_score


def True_block_merge(True_block):
    """
    :return: subject_start, query_start, merged_block
    """
    merged_block = ["", "", "", "", "", ""]
    for i in True_block:
        for j in range(0, 6):
            merged_block[j] = merged_block[j] + i[3][j]

    merged_block_clean = ["", "", "", "", "", ""]
    for i in range(0, len(merged_block[0])):
        if set([merged_block[j][i] for j in range(0, 6)]) == set(" "):
            continue
        else:
            for j in range(0, 6):
                merged_block_clean[j] = merged_block_clean[j] + merged_block[j][i]

    return True_block[0][1], True_block[0][2], merged_block_clean


def cds_parser(cds_block, position=1):
    line1, line2, line3, line4, line5, line6 = cds_block
    num = -1
    stop = []
    frame = []
    cds = ""
    protein = ""
    begin = position
    position = position - 1
    for i in line3:
        num = num + 1
        stop_flag = 0
        pseudo_flag = 0
        # print "\n"
        # print position
        # print i
        # print line4[num]+line5[num]+line6[num]
        if i == "-":  # GAP
            continue
        elif i == " " and line4[num] == " ":  # Null
            continue
        elif i == " " and (not line4[num] == " "):  # frame_intron
            position = position + 1
            cds = cds + line4[num]
        elif i == "X":  # stop code
            stop.append(position)
            position = position + 3
            cds = cds + line4[num] + line5[num] + line6[num]
            protein = protein + "X"
        elif i == "!":  # frame_pseudo
            frame.append(position)
            protein = protein + "X"
            cds = cds + "NNN"
            position = position + int(line4[num])
        else:  # normal
            position = position + 3
            protein = protein + i
            cds = cds + line4[num] + line5[num] + line6[num]

    return cds, protein, frame, stop, position - begin + 1


def intron_parser(merged_block_clean, block_subject_start, query_start):
    p_intron = re.compile('<(\d)-*\[(\d+) *\: *(\d+)\]-*(\d)>')
    line1, line2, line3, line4, line5, line6 = merged_block_clean

    if len(list(re.finditer(p_intron, line5))) == 0:
        block_dict = OrderedDict()
        last_intron_end = -1
        last_intron_end_site = block_subject_start - 1
        last_intron_phase = 0
        num = 0

        # make cds
        last_cds = ["", "", "", "", "", ""]
        num = num + 1
        for i in range(last_intron_end + 1, len(line5)):
            for j in range(0, 6):
                last_cds[j] = last_cds[j] + merged_block_clean[j][i]
        block_dict[num] = {}
        block_dict[num]["block"] = last_cds
        block_dict[num]['type'] = 'cds'
        block_dict[num]['phase'] = last_intron_phase
        block_dict[num]['start'] = last_intron_end_site + 1
        block_dict[num]['cds'], block_dict[num]['pro'], block_dict[num]['frame'], block_dict[num]['stop'], \
        block_dict[num]['true_length'] = cds_parser(
            block_dict[num]["block"], block_dict[num]['start'])

        whole_length = sum([block_dict[i]['true_length'] for i in block_dict])

        block_subject_end = block_subject_start + whole_length - 1
        block_dict[num]['end'] = block_subject_end

        # add query range
        last_q_end = query_start - 1
        for i in block_dict:
            if block_dict[i]['type'] == 'cds':
                block_dict[i]['q_start'] = last_q_end + 1
                block_dict[i]['q_end'] = block_dict[i]['q_start'] + len(block_dict[i]['block'][0].replace('-', '')) - 1
                last_q_end = block_dict[i]['q_end']


        return block_dict
    else:
        block_dict = OrderedDict()
        last_intron_end = -1
        last_intron_end_site = block_subject_start - 1
        last_intron_phase = 0
        num = 0
        for intron_match in re.finditer(p_intron, line5):
            phase, i_start, i_end, phase = intron_match.groups()
            i_start = int(i_start)
            i_end = int(i_end)
            phase = int(phase)

            # make cds
            last_cds = ["", "", "", "", "", ""]
            num = num + 1
            for i in range(last_intron_end + 1, intron_match.start()):
                for j in range(0, 6):
                    last_cds[j] = last_cds[j] + merged_block_clean[j][i]
            block_dict[num] = {}
            block_dict[num]["block"] = last_cds
            block_dict[num]['type'] = 'cds'
            block_dict[num]['phase'] = last_intron_phase
            block_dict[num]['start'] = last_intron_end_site + 1
            block_dict[num]['end'] = int(i_start) - 1
            block_dict[num]['cds'], block_dict[num]['pro'], block_dict[num]['frame'], block_dict[num]['stop'], \
            block_dict[num]['true_length'] = cds_parser(
                block_dict[num]["block"], block_dict[num]['start'])

            last_intron_end = intron_match.end() - 1
            last_intron_end_site = i_end

            # make intron
            intron_block = ["", "", "", "", "", ""]
            for i in range(intron_match.start(), intron_match.end() + 1):
                for j in range(0, 6):
                    intron_block[j] = intron_block[j] + merged_block_clean[j][i]

            num = num + 1
            block_dict[num] = {}
            block_dict[num]["block"] = intron_block
            block_dict[num]['type'] = 'intron'
            block_dict[num]['phase'] = 0
            last_intron_phase = (3 - phase) % 3
            # last_intron_phase = phase
            block_dict[num]['start'] = i_start
            block_dict[num]['end'] = i_end
            block_dict[num]['cds'] = ''
            block_dict[num]['true_length'] = block_dict[num]['end'] - block_dict[num]['start'] + 1
            match_obj = re.findall(r'(\S):(\S)\[(...)\]', block_dict[num]['block'][2])
            if len(match_obj) == 0 and block_dict[num]['phase'] != 0:
                raise ValueError('phase not as 0, but failed to found aa')
            if len(match_obj) != 0:
                block_dict[num]['pro'] = match_obj[0][1]
            else:
                block_dict[num]['pro'] = ""

        # make tail cds
        last_cds = ["", "", "", "", "", ""]
        num = num + 1
        for i in range(last_intron_end + 1, len(line5)):
            for j in range(0, 6):
                last_cds[j] = last_cds[j] + merged_block_clean[j][i]
        block_dict[num] = {}
        block_dict[num]["block"] = last_cds
        block_dict[num]['type'] = 'cds'
        block_dict[num]['phase'] = last_intron_phase
        block_dict[num]['start'] = last_intron_end_site + 1

        block_dict[num]['cds'], block_dict[num]['pro'], block_dict[num]['frame'], block_dict[num]['stop'], \
        block_dict[num][
            'true_length'] = cds_parser(block_dict[num]["block"], block_dict[num]['start'])

        whole_length = sum([block_dict[i]['true_length'] for i in block_dict])

        block_subject_end = block_subject_start + whole_length - 1
        block_dict[num]['end'] = block_subject_end

        # add query range
        last_q_end = query_start - 1
        for i in block_dict:
            if block_dict[i]['type'] == 'cds':
                block_dict[i]['q_start'] = last_q_end + 1
                block_dict[i]['q_end'] = block_dict[i]['q_start'] + len(block_dict[i]['block'][0].replace('-', '')) - 1
                last_q_end = block_dict[i]['q_end']

        return block_dict


def genewise_output_parser(genewise_out):
    with open(genewise_out, 'r') as f:
        all_text = f.read()
        info = all_text.split('//')
        # print(len(info))
        # if not len(info) == 5:
        #     raise ValueError("Error: Bad genewise output file\n")

    # align_sub
    align_sub = info[0].split('\n\n')
    True_block, query_name, subject_name, genewise_score = parse_align_sub(align_sub)

    # get query start end
    query_start = int(True_block[0][2])
    query_end_temp = int(True_block[-1][2])
    last_block_query = True_block[-1][3][0]
    last_block_query = re.sub(r'\s+', '', last_block_query)
    last_block_query = re.sub(r'-', '', last_block_query)
    query_end = query_end_temp + len(last_block_query) - 1

    # merge and parse block
    subject_start, query_start_block, merged_block = True_block_merge(True_block)

    if int(query_start) != int(query_start_block):
        raise ValueError("query start bug")

    block_dict = intron_parser(merged_block, int(subject_start), query_start)
    block_pro_seq = "".join([block_dict[i]['pro'] for i in block_dict]).upper()
    block_cds_seq = "".join([block_dict[i]['cds'] for i in block_dict]).upper()

    # protein_sub
    if len(info[2].split('>')) > 1:
        protein_seq = "".join(info[2].split('>')[1].split('\n')[1:])

    # gff_sub
    gff_sub_cds_dict = {}
    num = 0
    gff_output = []
    if len(info) > 4:
        for i in info[3].split('\n'):
            if i == '':
                continue
            gff_contig, gff_predict, gff_type, gff_start, gff_end, gff_score, gff_strand, gff_phase, gff_attr = i.split(
                '\t')
            if gff_type == 'cds' or gff_type == 'intron':
                num = num + 1
                gff_sub_cds_dict[num] = {}
                gff_sub_cds_dict[num]['type'] = gff_type
                if gff_phase == '.':
                    gff_phase = 0
                gff_sub_cds_dict[num]['phase'] = int(gff_phase)
                gff_sub_cds_dict[num]['start'] = int(gff_start)
                gff_sub_cds_dict[num]['end'] = int(gff_end)
            elif gff_type == 'match':
                gff_output = [gff_contig, float(gff_score), gff_attr, int(gff_start), int(gff_end), query_start,
                              query_end]
    else:
        info_list = info[1].split('\n')
        for i in range(0, len(info_list)):
            string_tmp = info_list[i]
            if string_tmp == '':
                continue
            gff_info = string_tmp.split()
            if gff_info[0] == 'Gene' and len(gff_info) < 3:
                continue
            elif gff_info[0] == 'Gene' and len(gff_info) >= 3:
                gff_output = [subject_name, genewise_score, None, int(gff_info[1]), int(gff_info[2]), query_start,
                              query_end]
            elif len(gff_info) == 5:
                num = num + 1
                gff_sub_cds_dict[num] = {}
                gff_sub_cds_dict[num]['type'] = 'cds'
                gff_sub_cds_dict[num]['phase'] = (3 - int(gff_info[4])) % 3
                gff_sub_cds_dict[num]['start'] = int(gff_info[1])
                gff_sub_cds_dict[num]['end'] = int(gff_info[2])

                if info_list[i + 1] == '':
                    break

                num = num + 1
                gff_sub_cds_dict[num] = {}
                gff_sub_cds_dict[num]['type'] = 'intron'
                gff_sub_cds_dict[num]['phase'] = 0
                gff_sub_cds_dict[num]['start'] = int(gff_info[2]) + 1
                gff_sub_cds_dict[num]['end'] = int(info_list[i + 1].split()[1]) - 1

    # block_check
    # phase have bug in genewise gff sub output, so we can't check it
    # for rank_id in block_dict:
    #     for check_type in ['start', 'end', 'type']:
    #         if not block_dict[rank_id][check_type] == gff_sub_cds_dict[rank_id][check_type]:
    #             raise ValueError("block check failed %d: %s" % (rank_id, check_type))
    # if not block_pro_seq == protein_seq:
    #     raise ValueError("block check failed protein seq %s" % genewise_out)

    return block_dict, gff_output, block_pro_seq, block_cds_seq


def block_dict_2_gff_rec(block_dict, gff_output, query_length):
    subject_id, score, hit_id, start, end, query_start, query_end = gff_output
    # make contig rec
    contig_rec = SeqRecord(Seq(""), "subject")

    # make gene rec
    qualifiers = {
        "source": "genewise",
        "score": score,
        "ID": "genewise_hit",
    }
    top_feature = SeqFeature(FeatureLocation(start - 1, end), type="gene", strand=1, qualifiers=qualifiers)

    # make mRNA rec
    mRNA_feature = SeqFeature(FeatureLocation(start - 1, end), strand=1)
    qualifiers = {
        "source": "genewise",
        "score": score,
        "ID": "genewise_hit.mRNA",
        "Target": "query",
        "Target_Start": query_start,
        "Target_End": query_end,
        "Target_Length": query_length
    }

    mRNA_feature.qualifiers = qualifiers
    mRNA_feature.sub_features = []
    mRNA_feature.type = "mRNA"
    top_feature.sub_features = [mRNA_feature]

    # make cds rec
    cds_num = 1
    pseudo_flag_gene = 0
    for rank_id in block_dict:
        if block_dict[rank_id]['type'] == 'cds':
            start = block_dict[rank_id]['start']
            end = block_dict[rank_id]['end']

            feature_now = SeqFeature(FeatureLocation(start - 1, end), strand=1)

            feature_now.type = 'CDS'
            qualifiers = {
                "source": "genewise",
                "score": 0.0,
                "phase": block_dict[rank_id]['phase'],
                "ID": "genewise_hit.cds" + str(cds_num),
                "Target": "query",
                "Target_Start": block_dict[rank_id]['q_start'],
                "Target_End": block_dict[rank_id]['q_end'],
                "Target_Length": query_length
            }
            feature_now.qualifiers = qualifiers

            if len(block_dict[rank_id]['frame']):
                feature_now.qualifiers["Note_frame_shift"] = []
                for j in block_dict[rank_id]['frame']:
                    feature_now.qualifiers["Note_frame_shift"].append(j)
                pseudo_flag_gene = 1
            if len(block_dict[rank_id]['stop']):
                feature_now.qualifiers["Note_stop_code"] = []
                for k in block_dict[rank_id]['stop']:
                    feature_now.qualifiers["Note_stop_code"].append(k)
                pseudo_flag_gene = 1

            top_feature.sub_features[0].sub_features.append(feature_now)

            cds_num = cds_num + 1

    # add pseudo flag
    if pseudo_flag_gene:
        top_feature.sub_features[0].qualifiers["Note"] = "pseudogene"

    contig_rec.features = [top_feature]

    return contig_rec


def gff_rec_obj_rename(gff_rec_object, hit_name="genewise_hit", q_name="query", s_name="subject", s_base=1, strand=1):
    gff_rec_object.id = s_name
    for gene in gff_rec_object.features:
        if strand == 1:
            abs_start = s_base + gene.location.start + 1 - 1
            abs_end = s_base + gene.location.end - 1
        else:
            abs_end = s_base - (gene.location.start + 1) + 1
            abs_start = s_base - gene.location.end + 1
        gene.location = FeatureLocation(abs_start - 1, abs_end)
        gene.strand = strand
        gene.id = hit_name
        gene.qualifiers['ID'] = hit_name
        for mRNA in gene.sub_features:
            if strand == 1:
                abs_start = s_base + mRNA.location.start + 1 - 1
                abs_end = s_base + mRNA.location.end - 1
            else:
                abs_end = s_base - (mRNA.location.start + 1) + 1
                abs_start = s_base - mRNA.location.end + 1
            mRNA.location = FeatureLocation(abs_start - 1, abs_end)
            mRNA.strand = strand
            mRNA.id = hit_name + ".mRNA"
            mRNA.qualifiers['Target'] = q_name
            mRNA.qualifiers['ID'] = mRNA.id
            cds_num = 0
            for cds in mRNA.sub_features:
                cds_num = cds_num + 1
                if strand == 1:
                    abs_start = s_base + cds.location.start + 1 - 1
                    abs_end = s_base + cds.location.end - 1
                else:
                    abs_end = s_base - (cds.location.start + 1) + 1
                    abs_start = s_base - cds.location.end + 1
                cds.location = FeatureLocation(abs_start - 1, abs_end)
                cds.strand = strand
                cds.id = hit_name + ".cds" + str(cds_num)
                cds.qualifiers['ID'] = cds.id
                cds.qualifiers['Target'] = q_name
                for pseudo_type in ["Note_stop_code", "Note_frame_shift"]:
                    if pseudo_type in cds.qualifiers:
                        tmp_new = []
                        for i in cds.qualifiers[pseudo_type]:
                            if strand == 1:
                                abs_site = s_base + int(i) - 1
                            else:
                                abs_site = s_base - int(i) + 1
                            tmp_new.append(abs_site)
                        cds.qualifiers[pseudo_type] = tmp_new

    return gff_rec_object


if __name__ == '__main__':

    subject_file = '/lustre/home/xuyuxing/Work/tmp/C000N_14332_25011.fa'
    query_file = '/lustre/home/xuyuxing/Work/tmp/C000N0003E0.1.aa'
    genewise_out = '/lustre/home/xuyuxing/Work/tmp/C000N0003E0.gws'
    q_name = "C000N0003E0"
    s_name = "C000N"
    h_name = "genewise_hit"
    s_base = 14332
    strand = "+"
    output_prefix = '/lustre/home/xuyuxing/Work/tmp/C000N0003E0.genewise'

    """
    subject_file = '/lustre/home/xuyuxing/Work/tmp/tmp/145ba845d80f4f2fafe174b70675df32.fr.sfa'
    query_file = '/lustre/home/xuyuxing/Work/tmp/tmp/145ba845d80f4f2fafe174b70675df32.qfa'
    genewise_out = '/lustre/home/xuyuxing/Work/tmp/tmp/145ba845d80f4f2fafe174b70675df32.fr.gws'
    output_prefix = '/lustre/home/xuyuxing/Work/tmp/tmp/145ba845d80f4f2fafe174b70675df32.fr.gwse'
    s_base = 1
    """

    genewise_run(query_file, subject_file, genewise_out, strand)
    block_dict, gff_output, block_pro_seq, block_cds_seq = genewise_output_parser(genewise_out)

    record_dict = SeqIO.to_dict(SeqIO.parse(query_file, "fasta"))
    query_length = len(str(record_dict[list(record_dict.keys())[0]].seq))
    contig_rec = block_dict_2_gff_rec(block_dict, gff_output, query_length)

    renamed_contig_rec = gff_rec_obj_rename(contig_rec, hit_name=h_name, q_name=q_name, s_name=s_name, s_base=s_base,
                                            strand=1)

    if not output_prefix is None:
        with open(output_prefix + ".cds", 'w') as f:
            f.write(">%s.cds\n%s" % (h_name, block_cds_seq))

        with open(output_prefix + ".pep", 'w') as f:
            f.write(">%s.pep\n%s" % (h_name, block_pro_seq))

        with open(output_prefix + ".gff3", "w") as f:
            GFF.write([renamed_contig_rec], f)



from src.xuyuxing.GenomeTools.WPGmapper import load_map_files, extract_all_evidences
from api.xuyuxing.file_parser.fileIO import tsv_file_parse
from lib.common.genome.genome_feature2 import HomoPredictResults
from datetime import datetime
from collections import Counter
from lib.common.math.interval import section, merge_intervals, overlap_between_interval_set, \
    group_by_intervals_with_overlap_threshold, sum_interval_length
from src.xuyuxing.tools.phytools import PtTree
from Bio import Phylo
import pickle
import sqlite3
import re
import os
from interlap import InterLap
from config import wu_blast_path
from BCBio import GFF
from api.xuyuxing.genome.mcl import gene_mcl
from lib.common.sqlite_command import sqlite_select
from lib.common.evolution.tree_operate import monophyly
from lib.common.util import printer_list
from lib.common.util import logging_init, configure_parser
from api.common.genome.blast import evalue_to_wvalue
from lib.common.genome.seq_base import read_fasta_by_faidx, extract_seq_from_sqlite, write_fasta
from lib.common.fileIO import tsv_file_dict_parse
from lib.common.os import mkdir, cmd_run, multiprocess_running
from api.xuyuxing.genome.genblasta import genblasta_run, genblasta_to_genewise, get_HPR_from_genewise_results, gene_to_aa, \
    gene_to_cds, query_gene_to_speci
from api.xuyuxing.comp_genome.orthofinder import OrthoFinderResults, get_species_id_from_gene_id
import configparser
import argparse
import sys


def if_conserve_and_target_speci_loss(input_GeneSet, target_speci, core_ref_list, norm_ref_list, tolerance_ratio=1.0):
    """
    To say if a gene set is good to say a target speci is lossed
    :param input_GeneSet: GeneSet object from orthofinder
    :param target_speci: "2"
    :param core_ref_list: ["0"]
    :param norm_ref_list: ["1","3","6","7","8","9"]
    :param tolerance_ratio: 0.83

    orthofinder_dir_path = "/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10"

    # parse orthofinder dir
    ortho_obj = OrthoFinderResults(orthofinder_dir_path)
    ortho_obj.get_Orthologues()

    input_GeneSet = ortho_obj.orthologues_dir['OG0000001_0']
    core_ref_list = ["0"]
    norm_ref_list = ["1","3","6","7","8","9"]
    target_speci = "2"
    tolerance_ratio = 0.83

    """
    speci_info_dir = input_GeneSet.speci_stat()
    conserve_flag = False
    target_loss_flag = False
    if (set(core_ref_list) & set(speci_info_dir.keys()) == set(core_ref_list)) and (len(
            set(norm_ref_list) & set(speci_info_dir.keys())) / len(norm_ref_list) >= tolerance_ratio):
        conserve_flag = True

    if conserve_flag and (target_speci not in speci_info_dir):
        target_loss_flag = True

    return conserve_flag, target_loss_flag


def get_hit_evalue_list(query_gene_name_list, db_file):

    content1 = sqlite_select(db_file, "query",
                             column_list=["q_hsp_id"], key_name="qDef",
                             value_tuple=query_gene_name_list)
    if len(content1) == 0:
        return []

    hsp_id_list = []
    for i in content1:
        q_hsp_id_str = i[0]
        for j in q_hsp_id_str.split(","):
            hsp_id_list.append(int(j))

    content2 = sqlite_select(db_file, "hsp",
                             column_list=["query_id", "hit_id", "Hsp_evalue"], key_name="hsp_id",
                             value_tuple=hsp_id_list)
    return content2


def get_homology_gene(query_gene_name_list, blast_results_file, log_file):

    args_list = []
    for blast_pair in blast_results_file:
        db_file = blast_results_file[blast_pair]['db']
        args_list.append((query_gene_name_list, db_file))

    mlt_outdir = multiprocess_running(
        get_hit_evalue_list, args_list, 30, log_file=log_file)

    query_gene_name_hash = {i: 0 for i in query_gene_name_list}

    homology_gene_dict = {}
    for i in mlt_outdir:
        hit_evalue_list_tmp = mlt_outdir[i]['output']
        for query_id, hit_id, Hsp_evalue in hit_evalue_list_tmp:

            if query_id not in homology_gene_dict:
                homology_gene_dict[query_id] = {}

            if hit_id not in homology_gene_dict[query_id]:
                homology_gene_dict[query_id][hit_id] = 0

            homology_gene_dict[query_id][hit_id] = homology_gene_dict[query_id][hit_id] + evalue_to_wvalue(
                float(Hsp_evalue))

    return homology_gene_dict


def tree_check_prepare(OL_gene_list, work_dir, homology_gene_dict, sequences_info, target_speci, core_ref, norm_ref,
                       genewise_dict, annotated_coverage=0.8, outgroup_seq_num=5, full_family_tree=False, threads=10):
    """
    OL_gene_list = OL_gene_list
    work_dir = work_dir
    blast_results_file = ortho_obj.blast_results_file
    sequences_info = ortho_obj.sequences_info
    target_speci = target_speci
    core_ref = core_ref
    norm_ref = norm_ref
    genewise_dict = genewise_dict
    gene_coverage = args.gene_coverage
    annotated_coverage = args.annotated_coverage
    outgroup_seq_num = args.top_outgroup_num
    full_family_tree = False
    """

    mkdir(work_dir, True)

    # get homo seq and new loci seq
    seq_info_file = work_dir + "/seq_info.pyb"
    if not os.path.exists(seq_info_file):
        # load HPR to every query

        # build homology_gene_dir
        query_gene_name_dict = {
            query_gene.id: 0 for query_gene in OL_gene_list}

        homology_gene_dict_in_this_tree = {}
        for q_id in query_gene_name_dict:
            if q_id in homology_gene_dict:
                for h_id in homology_gene_dict[q_id]:
                    if h_id in query_gene_name_dict:
                        continue
                    else:
                        if h_id not in homology_gene_dict_in_this_tree:
                            homology_gene_dict_in_this_tree[h_id] = 0
                        homology_gene_dict_in_this_tree[h_id] += homology_gene_dict_in_this_tree[h_id] + \
                            homology_gene_dict[q_id][h_id]

        # get homology gene
        if full_family_tree:
            homology_gene_list = [sequences_info[get_species_id_from_gene_id(i)][i] for i in
                                  homology_gene_dict]
        else:
            # get top wvalue for each speci
            homology_gene_name_list = list(query_gene_name_dict.keys())
            for speci in core_ref + norm_ref + [target_speci]:
                homology_gene_name_list.extend(
                    sorted([i for i in homology_gene_dict_in_this_tree if get_species_id_from_gene_id(i) == speci],
                           key=lambda x: homology_gene_dict_in_this_tree[x], reverse=True)[0:outgroup_seq_num])
            homology_gene_list = [sequences_info[get_species_id_from_gene_id(i)][i] for i in
                                  homology_gene_name_list]

        # build good_genewise_hit_dir
        good_genewise_hit_dir = {}
        tmp_num = 0
        for query_gene in OL_gene_list:
            tmp_num += 1
            # print(tmp_num)
            if query_gene.species in core_ref + norm_ref:
                # get genewise out from query
                # query should have genewise output
                if query_gene.id in genewise_dict:
                    HPR_tmp = genewise_dict[query_gene.id]
                    query_gene.HPR = HPR_tmp
                    for genewise_hit in query_gene.HPR.hit_gene_list:
                        # HPR should be good: good coverage and new annotation
                        if genewise_hit.map_quality_pass and genewise_hit.new_anno[0]:
                            # put HPR from diff query in chr strand index
                            genewise_hit.query_gene = query_gene
                            hit_chr_loci = genewise_hit.chr_loci
                            if hit_chr_loci.chr_id not in good_genewise_hit_dir:
                                good_genewise_hit_dir[hit_chr_loci.chr_id] = {
                                    "+": {}, "-": {}}
                            good_genewise_hit_dir[hit_chr_loci.chr_id][hit_chr_loci.strand][
                                genewise_hit] = genewise_hit.chr_loci.range

        # merge the same loci, get a new seq file
        new_gene_list = []
        for contig in good_genewise_hit_dir:
            for strand in good_genewise_hit_dir[contig]:
                good_genewise_hit_dir[contig][strand] = group_by_intervals_with_overlap_threshold(
                    good_genewise_hit_dir[contig][strand], annotated_coverage)

                for group_id in good_genewise_hit_dir[contig][strand]:
                    genewise_hit_list = good_genewise_hit_dir[contig][strand][group_id]['list']
                    lengthest_hit = sorted(genewise_hit_list, key=lambda x: len(x.model_aa_seq.seq), reverse=True)[
                        0]
                    good_genewise_hit_dir[contig][strand][group_id]['rep_hit'] = lengthest_hit
                    new_gene_list.append(lengthest_hit)

        # subject will let no ref gene into homology list
        homology_gene_list_filter = [
            i for i in homology_gene_list if i.species in core_ref + norm_ref + [target_speci]]

        pickle_info = (homology_gene_list_filter, new_gene_list)

        OUT = open(seq_info_file, 'wb')
        pickle.dump(pickle_info, OUT)
        OUT.close()


def tree_check(OL_id, OL_gene_list, work_dir, species_info, target_speci, target_speci_raw_name,
               query_in_mcl_cluster_ratio=0.8, support_threshold=0.8, jaccard_threshold=0.8, full_family_tree=False,
               cpu_num=1):
    query_gene_name_list = [query_gene.id for query_gene in OL_gene_list]

    # get homo seq and new loci seq
    seq_info_file = work_dir + "/seq_info.pyb"
    TEMP = open(seq_info_file, 'rb')
    homology_gene_list, new_gene_list = pickle.load(TEMP)
    TEMP.close()

    # if no outgroup
    if len(homology_gene_list) == len(query_gene_name_list):
        # make a report for seq
        OL_seq_report_file = work_dir + "/seq_report.txt"
        with open(OL_seq_report_file, 'w') as f:
            f.write("Seq_ID\tSeq_raw_ID\tSpeci\tSpeci_name\tSeq_type\tSeq\n")
            for query_gene in OL_gene_list:
                print_string = printer_list(
                    [query_gene.id, query_gene.old_name['raw_short_id'], query_gene.species,
                     species_info[query_gene.species]['id'], "query",
                     query_gene.model_aa_seq])
                f.write(print_string + "\n")
            for gene in homology_gene_list:
                if gene not in OL_gene_list:
                    print_string = printer_list([gene.id, gene.old_name['raw_short_id'], gene.species,
                                                 species_info[gene.species]['id'], "homology",
                                                 gene.model_aa_seq])
                    f.write(print_string + "\n")

            for gene in new_gene_list:
                print_string = printer_list([gene.id, None, target_speci, target_speci_raw_name, "genewise",
                                             gene.model_aa_seq])
                f.write(print_string + "\n")

        # if len(new_gene_list) == 0:
        #     return True, "No new genewise"
        # elif len(homology_gene_list) == len(query_gene_name_list):
        #     return False, "No outgroup seq and have new genewise"

        if len(new_gene_list) != 0:
            return False, "No outgroup and have new genewise"
        else:
            return True, "No outgroup and no new genewise"

    else:
        # make tree
        tree_dir = work_dir + "/tree"
        tree_output_dir = tree_dir + "/tree_out"
        tree_file = tree_output_dir + "/tree_out.phb"

        if not os.path.exists(tree_file):
            if full_family_tree:
                # make mcl to get gene cluster
                mcl_info_file = work_dir + "/mcl_info.pyb"
                if not os.path.exists(mcl_info_file):
                    all_gene_list = homology_gene_list + new_gene_list
                    mcl_dir = work_dir + "/mcl"
                    mkdir(mcl_dir, True)
                    mcl_group_dir = gene_mcl(all_gene_list, cpu_num=cpu_num, work_dir=mcl_dir, keep_flag=True,
                                             diamond_flag=True)
                    mcl_group_hash = {}
                    for i in mcl_group_dir:
                        for j in mcl_group_dir[i]:
                            mcl_group_hash[j] = i

                    # found cluster which have query seq
                    best_cluster = sorted(mcl_group_dir,
                                          key=lambda x: len(
                                              set([i.id for i in OL_gene_list]) & set(mcl_group_dir[x])) / len(
                                              OL_gene_list), reverse=True)[0]
                    if len(set([i.id for i in OL_gene_list]) & set(mcl_group_dir[best_cluster])) / len(
                            OL_gene_list) < query_in_mcl_cluster_ratio:
                        OL_cluster = False

                    # make a report for seq
                    OL_seq_report_file = work_dir + "/seq_report.txt"
                    with open(OL_seq_report_file, 'w') as f:
                        f.write(
                            "Seq_ID\tSeq_raw_ID\tSpeci\tSpeci_name\tSeq_type\tCluster\tSeq\n")
                        for query_gene in OL_gene_list:
                            if query_gene.id in mcl_group_hash:
                                cluster_id = mcl_group_hash[query_gene.id]
                            else:
                                cluster_id = None
                            print_string = printer_list(
                                [query_gene.id, query_gene.old_name['raw_short_id'], query_gene.species,
                                 species_info[query_gene.species]['id'], "query",
                                 cluster_id, query_gene.model_aa_seq])
                            f.write(print_string + "\n")
                        for gene in homology_gene_list:
                            if gene not in OL_gene_list:
                                if gene.id in mcl_group_hash:
                                    cluster_id = mcl_group_hash[gene.id]
                                else:
                                    cluster_id = None
                                print_string = printer_list([gene.id, gene.old_name['raw_short_id'], gene.species,
                                                             species_info[gene.species]['id'], "homology",
                                                             cluster_id, gene.model_aa_seq])
                                f.write(print_string + "\n")

                        for gene in new_gene_list:
                            if gene.id in mcl_group_hash:
                                cluster_id = mcl_group_hash[gene.id]
                            else:
                                cluster_id = None
                            print_string = printer_list(
                                [gene.id, None, target_speci, target_speci_raw_name, "genewise",
                                 cluster_id, gene.model_aa_seq])
                            f.write(print_string + "\n")

                    # store pickle
                    pickle_info = (mcl_group_dir, best_cluster)

                    OUT = open(mcl_info_file, 'wb')
                    pickle.dump(pickle_info, OUT)
                    OUT.close()
                else:
                    TEMP = open(mcl_info_file, 'rb')
                    mcl_group_dir, best_cluster = pickle.load(TEMP)
                    TEMP.close()

                # make tree
                all_gene_list = homology_gene_list + new_gene_list
                all_gene_dir = {i.id: i for i in all_gene_list}
                tree_dir = work_dir + "/tree"
                mkdir(tree_dir, True)
                best_cluster_seq_file = tree_dir + "/tree.faa"
                with open(best_cluster_seq_file, 'w') as f:
                    for seq_name in mcl_group_dir[best_cluster]:
                        f.write(">%s\n%s\n" %
                                (seq_name, all_gene_dir[seq_name].model_aa_seq))

                PtTree(best_cluster_seq_file, 'fasttree', False,
                       tree_output_dir, cpu_num, "clustalo")

            else:
                # make tree
                all_gene_list = homology_gene_list + new_gene_list
                all_gene_dir = {i.id: i for i in all_gene_list}
                tree_dir = work_dir + "/tree"
                mkdir(tree_dir, True)
                top_tree_seq_file = tree_dir + "/tree.faa"
                with open(top_tree_seq_file, 'w') as f:
                    for seq_name in all_gene_dir:
                        f.write(">%s\n%s\n" %
                                (seq_name, all_gene_dir[seq_name].model_aa_seq))

                PtTree(top_tree_seq_file, 'fasttree', False,
                       tree_output_dir, cpu_num, "clustalo")

        # tree check
        query_gene_name_list = [query_gene.id for query_gene in OL_gene_list]
        new_gene_name_list = [gene.id for gene in new_gene_list]
        target_speci_gene_name_list = [
            gene.id for gene in homology_gene_list if gene.species == target_speci]

        tree = Phylo.read(tree_file, 'newick')
        good_topo, monophyly_leaf = monophyly(query_gene_name_list, tree, support_threshold=support_threshold,
                                              jaccard_threshold=jaccard_threshold, exclude_leaf_set=new_gene_name_list)

        added_new_gene = list(set(new_gene_name_list) & set(monophyly_leaf)) + list(
            set(target_speci_gene_name_list) & set(monophyly_leaf))

        # make a report for seq
        if good_topo:
            Tree_Check = "good_topo"
        else:
            Tree_Check = "bad_topo"

        OL_seq_report_file = work_dir + "/seq_report.txt"
        with open(OL_seq_report_file, 'w') as f:
            f.write(
                "Seq_ID\tSeq_raw_ID\tSpeci\tSpeci_name\tSeq_type\tTree_Check\tSeq\n")
            for query_gene in OL_gene_list:
                print_string = printer_list(
                    [query_gene.id, query_gene.old_name['raw_short_id'], query_gene.species,
                     species_info[query_gene.species]['id'], "query", Tree_Check,
                     query_gene.model_aa_seq])
                f.write(print_string + "\n")
            for gene in homology_gene_list:
                if gene not in OL_gene_list:
                    if gene.id in added_new_gene:
                        print_string = printer_list(
                            [gene.id, gene.old_name['raw_short_id'], gene.species, species_info[gene.species]['id'],
                             "homology", "add", gene.model_aa_seq])
                    else:
                        print_string = printer_list([gene.id, gene.old_name['raw_short_id'], gene.species,
                                                     species_info[gene.species]['id'], "homology", "None",
                                                     gene.model_aa_seq])
                    f.write(print_string + "\n")

            for gene in new_gene_list:
                if gene.id in added_new_gene:
                    print_string = printer_list(
                        [gene.id, None, target_speci, target_speci_raw_name, "genewise", "add",
                         gene.model_aa_seq])
                else:
                    print_string = printer_list(
                        [gene.id, None, target_speci, target_speci_raw_name, "genewise", "None",
                         gene.model_aa_seq])
                f.write(print_string + "\n")

        if good_topo:
            if len(new_gene_list) == 0:
                return True, "Good topo and no added gene"

            if len(added_new_gene):
                return False, added_new_gene
            else:
                return True, "Good topo and no added gene"
        else:
            if len(new_gene_list) == 0:
                return True, "Bad topo but no new gene"
            else:
                return False, "Bad topo"


def get_query_id_from_HPR_id(HPR_id):
    return re.sub('_\d+$', '', HPR_id)


def mark_map_quality_for_gf_dict(input_gf_dict, args):
    for ev_id in input_gf_dict:
        # quality control
        coverage_flag = input_gf_dict[ev_id].evidence_indicator["query_coverage"] >= args.min_cover
        aln_aa_len_flag = input_gf_dict[ev_id].evidence_indicator["aln_aa_len"] >= args.min_aa_len
        identity_flag = input_gf_dict[ev_id].evidence_indicator["identity"] >= args.min_identity
        score_flag = input_gf_dict[ev_id].evidence_indicator["score"] >= args.min_score
        # parent_evalue_flag = input_gf_dict[ev_id].parent_blast["evalue"] <= args.evalue
        # parent_blast_identity_flag = input_gf_dict[ev_id].parent_blast["identity"] >= args.min_identity

        # Special arrow
        # special_arrow_flag = input_gf_dict[ev_id].evidence_indicator["query_coverage"] >= 0.8 and input_gf_dict[ev_id].evidence_indicator["identity"] >= 0.9 and input_gf_dict[
        #     ev_id].parent_blast["identity"] >= 0.9 and input_gf_dict[ev_id].evidence_indicator["aln_aa_len"] >= 200 and input_gf_dict[ev_id].parent_blast["evalue"] <= 1e-30

        # if coverage_flag and aln_aa_len_flag and identity_flag and score_flag and parent_evalue_flag and parent_blast_identity_flag:
        if coverage_flag and aln_aa_len_flag and identity_flag and score_flag:
            input_gf_dict[ev_id].map_quality_pass = True
        # elif score_flag and special_arrow_flag:
        #     input_gf_dict[ev_id].map_quality_pass = True
        else:
            input_gf_dict[ev_id].map_quality_pass = False

    return input_gf_dict


def get_HPR_from_evidence_dict(query_gene_dict, evidence_dict, ref_WPGmapper_dict, subject_species=None):
    hit_gene_dir = {}
    for gf_id in evidence_dict:
        gf = evidence_dict[gf_id]
        query_id = gf.evidence_indicator['query_id']

        if not query_id in hit_gene_dir:
            hit_gene_dir[query_id] = []
        hit_gene_dir[query_id].append(gf)

    output_list = []
    for query_id in hit_gene_dir:
        query_gene = query_gene_dict[query_id]
        output_list.append(
            HomoPredictResults(query_gene, subject_species=subject_species, hit_gene_list=hit_gene_dir[query_id]))

    return output_list


def GeneLoss_main(args):
    mkdir(args.tmp_path, True)

    logger = logging_init("GeneLoss", args.log_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    # # todo next row is test
    # return True

    logger.info("Step1: parse orthofinder dir")
    ortho_obj = OrthoFinderResults(args.orthofinder_dir_path)

    # get file and dir info
    if not hasattr(args, 'modify_dict'):
        args.modify_dict = {}

    ortho_obj.pase_file_and_dir(modify_dict=args.modify_dict)

    # get orthologues or orthogroups
    ortho_obj.get_name_info(hash_table=True)
    if args.ortho_level == 'orthologues':
        ortho_obj.get_Orthologues()
        group_dir = ortho_obj.orthologues_dir

    elif args.ortho_level == 'orthogroups':
        ortho_obj.get_OrthoGroups()
        group_dir = ortho_obj.orthogroup_dir

    ortho_obj.read_aa_fasta()
    ortho_obj.blast_to_sqlite(threads=args.num_threads)

    # get candidate gene loss orthologues
    logger.info("Step2: get candidate gene loss orthologues")
    target_speci = [
        i for i in ortho_obj.species_info if ortho_obj.species_info[i]['id'] == args.target_speci][0]
    core_ref = [i for i in ortho_obj.species_info if ortho_obj.species_info[i]
                ['id'] in args.core_ref.split(',')]
    norm_ref = [i for i in ortho_obj.species_info if ortho_obj.species_info[i]
                ['id'] in args.norm_ref.split(',')]

    candidate_gene_loss = args.output_dir + "/1.candidate_gene_loss.txt"
    candi_loss_list = []
    conserved_OL_list = []

    if not os.path.exists(candidate_gene_loss):
        logger.info("first running Step2")
        for OL_id in group_dir:
            conserve_flag, target_loss_flag = if_conserve_and_target_speci_loss(group_dir[OL_id],
                                                                                target_speci,
                                                                                core_ref, norm_ref,
                                                                                args.tolerance_ratio)
            group_dir[OL_id].target_loss_flag = target_loss_flag
            group_dir[OL_id].conserve_flag = conserve_flag
            if target_loss_flag:
                candi_loss_list.append(OL_id)
            if conserve_flag:
                conserved_OL_list.append(OL_id)

        print_speci_list = args.core_ref.split(
            ',') + args.norm_ref.split(',') + args.target_speci.split(',')
        print_speci_id_list = []
        for i in print_speci_list:
            print_speci_id_list.append(
                [j for j in ortho_obj.species_info if ortho_obj.species_info[j]['id'] == i][0])

        with open(candidate_gene_loss, 'w') as f:
            f.write(printer_list(print_speci_list, head='OL_id\t') + "\n")
            for OL_id in conserved_OL_list:
                gene_set = group_dir[OL_id]
                speci_stat_dir = gene_set.speci_stat()
                gene_num_list = []
                for i in print_speci_id_list:
                    if i in speci_stat_dir:
                        gene_num_list.append(speci_stat_dir[i])
                    else:
                        gene_num_list.append(0)
                f.write(printer_list(gene_num_list, head=OL_id + "\t") + "\n")
    else:
        logger.info(
            "find 1.candidate_gene_loss.txt, get gene loss list from this file")
        gene_loss_dict = tsv_file_dict_parse(candidate_gene_loss)
        for i in gene_loss_dict:
            conserved_OL_list.append(gene_loss_dict[i]['OL_id'])
            if int(gene_loss_dict[i][args.target_speci]) == 0:
                candi_loss_list.append(gene_loss_dict[i]['OL_id'])

    logger.info("Step2: get %d conserved orthologues in which %d may lost in %s" % (
        len(conserved_OL_list), len(candi_loss_list), args.target_speci))

    # # todo next row is test
    # candi_loss_list = candi_loss_list[0:10]

    # get sequence of loss ortho
    logger.info("Step3: check if annotation error make gene loss in %s" %
                args.target_speci)

    # test if need redo
    genewise_dict_file = args.output_dir + "/2.genewise_dict.pyb"
    if not os.path.exists(genewise_dict_file):
        logger.info("Step3.1: load data from WPGmapper")

        speci_id_map = {}
        if hasattr(args, 'orthofinder_wpgmapper_id_map'):
            file_info = tsv_file_parse(args.orthofinder_wpgmapper_id_map)
            for i in file_info:
                OF_id, WPG_id = file_info[i]
                speci_id_map[OF_id] = WPG_id

        query_gene_dict = {}
        ref_gene_in_candi_loss_OL = []
        WPGmapper_species_id_list = []
        for OL_id in candi_loss_list:
            gene_set = group_dir[OL_id]
            for gene in gene_set.gene_list:
                if gene.species in core_ref + norm_ref:
                    query_gene_dict[gene.old_name['raw_short_id']] = gene

                    ref_gene_in_candi_loss_OL.extend(
                        [gene.old_name['raw_short_id']+"_"+str(i) for i in range(args.top_evidence_num)])

                    species_old_id = ortho_obj.species_info[gene.species]['id']
                    if species_old_id in speci_id_map:
                        species_old_id = speci_id_map[species_old_id]

                    if species_old_id not in WPGmapper_species_id_list:
                        WPGmapper_species_id_list.append(species_old_id)

        ref_WPGmapper_dict = load_map_files(
            WPGmapper_species_id_list, args.wpgmapper_dir)
        evidence_dict = extract_all_evidences(
            ref_WPGmapper_dict, evidence_id_list=ref_gene_in_candi_loss_OL, log_file=args.log_file)
        evidence_dict = mark_map_quality_for_gf_dict(evidence_dict, args)

        logger.info("Step3.1: begin compare")
        # load genewise output
        genewise_out = get_HPR_from_evidence_dict(
            query_gene_dict, evidence_dict, ref_WPGmapper_dict, subject_species=target_speci)

        # load annotated range
        annotated_range_dict = {}
        gene_cds_intervals_dict = {}
        with open(args.target_speci_annotation, 'r') as in_handle:
            for rec in GFF.parse(in_handle):
                if rec.id not in annotated_range_dict:
                    annotated_range_dict[rec.id] = {}
                    annotated_range_dict[rec.id]["+"] = InterLap()
                    annotated_range_dict[rec.id]["-"] = InterLap()
                for gene in rec.features:
                    if args.target_speci_feature == 'None' or gene.type == args.target_speci_feature:
                        start_tmp = gene.location.start.position + 1
                        end_tmp = gene.location.end.position
                        if gene.id == "":
                            gene.id = list(
                                set(gene.qualifiers.keys()) - {'source'})[0]
                        if gene.strand == 1:
                            annotated_range_dict[rec.id]["+"].add(
                                (start_tmp, end_tmp, gene.id))
                        else:
                            annotated_range_dict[rec.id]["-"].add(
                                (start_tmp, end_tmp, gene.id))

                        gene_cds_intervals_dict[gene.id] = [(int(i.location.start)+1, int(
                            i.location.end)) for i in gene.sub_features[0].sub_features if i.type == 'CDS']

        # mark new range which not in annotated range and have good coverage
        for query_HPR in genewise_out:
            for hit_gene in query_HPR.hit_gene_list:
                cds_range = [(i.start, i.end)
                             for i in hit_gene.sub_features[0].sub_features if i.type == 'CDS']
                cds_length = sum_interval_length(cds_range)

                mRNA_qual = hit_gene.sub_features[0].qualifiers

                # test if have been annotated
                loci_tmp = hit_gene.chr_loci
                hit_gene.new_anno = (True, None, 0.0)

                overlaped_gene_id = []

                if loci_tmp.chr_id in annotated_range_dict:
                    for annotated_range in annotated_range_dict[loci_tmp.chr_id][loci_tmp.strand].find(
                            (loci_tmp.start, loci_tmp.end)):
                        anno_s = annotated_range[0]
                        anno_e = annotated_range[1]
                        known_gene_id = annotated_range[2]
                        overlaped_gene_id.append(known_gene_id)

                overlaped_gene_id = list(set(overlaped_gene_id))

                for gene_id in overlaped_gene_id:
                    gene_cds_range = gene_cds_intervals_dict[gene_id]
                    overlap_ratio, overlap_length, overlap = overlap_between_interval_set(
                        cds_range, gene_cds_range)
                    overlap_ratio = overlap_length/cds_length

                    if overlap_ratio >= args.annotated_coverage:
                        hit_gene.new_anno = (
                            False, known_gene_id, overlap_ratio)
                    else:
                        hit_gene.new_anno = (
                            True, known_gene_id, overlap_ratio)

        # turn genewise_out to a dict, key is query_name
        genewise_dict = {}
        for query_HPR in genewise_out:
            genewise_dict[query_HPR.query_gene.id] = query_HPR

        OUT = open(genewise_dict_file, 'wb')
        pickle.dump(genewise_dict, OUT)
        OUT.close()
    else:
        logger.info("comparsion already finished")
        TEMP = open(genewise_dict_file, 'rb')
        genewise_dict = pickle.load(TEMP)
        TEMP.close()

    logger.info(
        "%d query seq get %d genewise hit, %d can pass coverage test, %d are on the known gene loci, %d are good new hit" % (
            len(genewise_dict),
            sum([len(genewise_dict[i].hit_gene_list) for i in genewise_dict]),
            sum([len([j for j in genewise_dict[i].hit_gene_list if j.map_quality_pass]) for i in
                 genewise_dict]),
            sum([len([j for j in genewise_dict[i].hit_gene_list if j.map_quality_pass and not j.new_anno[0]])
                 for i in genewise_dict]),
            sum([len(
                [j for j in genewise_dict[i].hit_gene_list if j.map_quality_pass and j.new_anno[0]])
                for i in genewise_dict])
        ))

    logger.info("Step3.4: tree check for gene loss orthologous")
    tree_check_pickle = args.output_dir + "/3.tree_check_pickle.pyb"
    if not os.path.exists(tree_check_pickle):
        logger.info(
            "Step3.4.1: make dir for every orthologous, and prepare new seq and old seq")
        tree_check_dir = args.tmp_path + "/tree_check"
        mkdir(tree_check_dir, True)

        homology_gene_dict = get_homology_gene(
            list(genewise_dict.keys()), ortho_obj.blast_results_file, args.log_file)

        args_list = []
        num = 0
        for OL_id in candi_loss_list:
            num = num + 1
            logger.info("%d/%d parse OL: %s" %
                        (num, len(candi_loss_list), OL_id))
            OL_dir = tree_check_dir + "/" + OL_id
            work_dir = OL_dir + "/o%ds%.1fj%.1f" % (
                args.top_outgroup_num, args.support_threshold, args.jaccard_threshold)
            OL_obj = group_dir[OL_id]
            OL_gene_list = [
                i for i in OL_obj.gene_list if i.species in core_ref + norm_ref + [target_speci]]

            tree_check_prepare(OL_gene_list, work_dir, homology_gene_dict, ortho_obj.sequences_info,
                               target_speci, core_ref, norm_ref, genewise_dict, args.annotated_coverage, args.top_outgroup_num, False, args.num_threads)

            args_list.append((OL_id, OL_gene_list, work_dir, ortho_obj.species_info, target_speci, args.target_speci,
                              args.query_in_mcl_cluster_ratio, args.support_threshold, args.jaccard_threshold, False,
                              1))

        logger.info("Step3.4.2: running tree check")
        cmd_result = multiprocess_running(
            tree_check, args_list, args.num_threads, args.log_file, True)

        tree_check_output = {}
        for i in cmd_result:
            tree_check_output[cmd_result[i]['args']
                              [0]] = cmd_result[i]['output']

        OUT = open(tree_check_pickle, 'wb')
        pickle.dump(tree_check_output, OUT)
        OUT.close()
    else:
        logger.info("comparsion already finished")
        TEMP = open(tree_check_pickle, 'rb')
        tree_check_output = pickle.load(TEMP)
        TEMP.close()

    logger.info("Step4: make report")

    # summary report
    summary_report_file = args.output_dir + "/4.summary_report.txt"

    with open(summary_report_file, 'w') as f:
        # head
        f.write("GeneLoss report writing in %s\n\n" % str(datetime.now()))

        # Arg list
        f.write("\nArgument list: \n--------------------------------\n")
        args_info_string = ""
        attrs = vars(args)

        for item in attrs.items():
            args_info_string = args_info_string + ("    %s: %s\n" % item)

        f.write(args_info_string)

        # ortho sum
        f.write("\nOrthologues: \n--------------------------------\n")
        f.write("Conserved / Total orthologues: %d / %d = %.2f%%\n" % (
            len(conserved_OL_list), len(group_dir),
            len(conserved_OL_list) / len(group_dir) * 100))

        gene_speci_in_OL = []
        gene_speci_in_conser_OL = []
        for OL_id in group_dir:
            gene_set_now = group_dir[OL_id]
            speci_list = [gene.species for gene in gene_set_now.gene_list]
            gene_speci_in_OL.extend(speci_list)
            if OL_id in conserved_OL_list:
                gene_speci_in_conser_OL.extend(speci_list)

        gene_speci_in_OL_counter = Counter(gene_speci_in_OL)
        gene_speci_in_conser_OL_counter = Counter(gene_speci_in_conser_OL)

        for i in core_ref + norm_ref + [target_speci]:
            f.write("    %s: %d / %d = %.2f%%\n" % (ortho_obj.species_info[i]['id'],
                                                    gene_speci_in_conser_OL_counter[i], gene_speci_in_OL_counter[i],
                                                    gene_speci_in_conser_OL_counter[i] / gene_speci_in_OL_counter[
                                                        i] * 100))

        f.write("\nCandidate gene loss: %d / %d = %.2f%%\n" % (
            len(candi_loss_list), len(conserved_OL_list),
            len(candi_loss_list) / len(conserved_OL_list) * 100))

        # tree test

        all_true = len(
            [i for i in tree_check_output if tree_check_output[i][0]])
        good_topo_no_new_gene = len(
            [i for i in tree_check_output if tree_check_output[i][1] == 'Good topo and no added gene'])
        no_outgroup_no_new_gene = len(
            [i for i in tree_check_output if tree_check_output[i][1] == 'No outgroup and no new genewise'])
        bad_topo_no_new_gene = len(
            [i for i in tree_check_output if tree_check_output[i][1] == 'Bad topo but no new gene'])

        all_false = len(
            [i for i in tree_check_output if not tree_check_output[i][0]])
        good_topo_new_gene = len(
            [i for i in tree_check_output if isinstance(tree_check_output[i][1], list)])
        bad_topo = len(
            [i for i in tree_check_output if tree_check_output[i][1] == 'Bad topo'])
        no_outgroup_new_gene = len(
            [i for i in tree_check_output if tree_check_output[i][1] == 'No outgroup and have new genewise'])

        f.write("\nTree test: \n--------------------------------\n")
        f.write("There are %d Orthologues group may have gene loss, in which:\n" % len(
            candi_loss_list))
        f.write("%d pass tree test, because:\n" % all_true)
        f.write("    %d have good topology and no new genewise hit can be cluster into clade\n" %
                good_topo_no_new_gene)
        f.write("    %d not found outgroup seq and not found new genewise hit\n" %
                no_outgroup_no_new_gene)
        f.write("    %d have a bad topology in tree and not found new genewise hit\n" %
                bad_topo_no_new_gene)
        f.write("%d not pass tree test, because:\n" % all_false)
        f.write(
            "    %d have good topology and have some new genewise hit can be cluster into clade\n" % good_topo_new_gene)
        f.write("    %d have a bad topology in tree\n" % bad_topo)
        f.write("    %d not found outgroup seq but can found new genewise hit\n" %
                no_outgroup_new_gene)

        f.write("\nPassed gene loss: %d / %d = %.2f%%\n" % (
            all_true, len(conserved_OL_list),
            all_true / len(conserved_OL_list) * 100))

    # table report
    table_report_file = args.output_dir + "/4.table_report.txt"

    print_speci_list = args.core_ref.split(
        ',') + args.norm_ref.split(',') + args.target_speci.split(',')
    print_speci_id_list = []
    for i in print_speci_list:
        print_speci_id_list.append(
            [j for j in ortho_obj.species_info if ortho_obj.species_info[j]['id'] == i][0])

    with open(table_report_file, 'w') as f:
        head_line_list = ['OL_id', 'conserved', 'gene_loss',
                          'gene_loss_detail'] + print_speci_list + print_speci_list
        f.write(printer_list(head_line_list) + "\n")

        for OL_id in conserved_OL_list:

            print_line_list = [OL_id, 'True']
            if OL_id in tree_check_output:
                if OL_id in [i for i in tree_check_output if tree_check_output[i][0]]:
                    print_line_list.extend(
                        ['True', tree_check_output[OL_id][1]])
                else:
                    print_line_list.append('False')
                    if isinstance(tree_check_output[OL_id][1], list):
                        loss_detail = printer_list(
                            tree_check_output[OL_id][1], sep=', ')
                    else:
                        loss_detail = tree_check_output[OL_id][1]
                    print_line_list.append(loss_detail)
            else:
                print_line_list.extend(['False', 'Not in candidate list'])

            gene_set = group_dir[OL_id]
            speci_stat_dir = gene_set.speci_stat()
            gene_num_list = []
            for i in print_speci_id_list:
                if i in speci_stat_dir:
                    gene_num_list.append(speci_stat_dir[i])
                else:
                    gene_num_list.append(0)

            print_line_list.extend(gene_num_list)

            gene_set_by_speci = {}
            for i in print_speci_id_list:
                gene_set_by_speci[i] = []
            for gene in gene_set.gene_list:
                if gene.species in print_speci_id_list:
                    gene_set_by_speci[gene.species].append(
                        gene.old_name['raw_short_id'])

            print_line_list.extend(
                [printer_list(gene_set_by_speci[i], sep=', ') for i in print_speci_id_list])

            f.write(printer_list(print_line_list) + "\n")

    # new gene gff and sequences
    genewise_hit_in_added_id_list = [tree_check_output[i][1] for i in tree_check_output if
                                     isinstance(tree_check_output[i][1], list)]

    tmp = []
    for i in genewise_hit_in_added_id_list:
        tmp.extend(i)

    genewise_hit_in_added_id_list = tmp

    table_report_file = args.output_dir + "/4.table_report.txt"

    # todo

    return tree_check_output


def GeneLoss_args_parser(args):

    # configure file argument parse

    try:
        script_dir_path = os.path.split(os.path.realpath(__file__))[0]
        defaults_config_file = script_dir_path + \
            "/GeneLoss_defaults.ini"
    except:
        defaults_config_file = "/lustre/home/xuyuxing/python_project/Genome_work_tools/src/xuyuxing/GenomeTools/GeneLoss_defaults.ini"
    # print(defaults_config_file)

    args_type = {
        # str
        "config_file": "str",
        "target_speci": "str",
        "target_speci_genome": "str",
        "target_speci_annotation": "str",
        "target_speci_feature": "str",
        "wpgmapper_dir": "str",
        "core_ref": "str",
        "norm_ref": "str",
        "log_file": "str",
        "orthofinder_dir_path": "str",
        "output_dir": "str",
        "tmp_path": "str",
        "ortho_level": "str",
        "orthofinder_wpgmapper_id_map": "str",

        # float
        "tolerance_ratio": "float",
        "annotated_coverage": "float",
        "query_in_mcl_cluster_ratio": "float",
        "support_threshold": "float",
        "jaccard_threshold": "float",
        "min_cover": "float",
        "min_identity": "float",

        # int
        "num_threads": "int",
        "top_evidence_num": "int",
        "min_aa_len": "int",
        "min_score": "int",
        "top_outgroup_num": "int",
    }

    args = configure_parser(args, defaults_config_file,
                            args.config_file, args_type, None)

    # Modify_dict parse
    cfg = configparser.ConfigParser()
    cfg.read(args.config_file)

    if "Modify_dict" in cfg.sections():
        args.modify_dict = {}
        for key in cfg["Modify_dict"]:
            value = cfg["Modify_dict"][key]
            args.modify_dict[key] = value

    # geneloss running
    tree_check_output = GeneLoss_main(args)


if __name__ == '__main__':
    # command argument parse
    class abc(object):
        pass

    args = abc()

    args.target_speci_genome = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Cau_WPGmapper/T267555N0.genome.fasta"
    args.target_speci = "T267555N0.gene_model.protein"
    args.target_speci_annotation = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Cuscuta_australis/T267555N0.genome.gff3"
    args.target_speci_feature = "gene"
    args.wpgmapper_dir = '/lustre/home/xuyuxing/Work/Gel/WPGmapper/Cau_WPGmapper'
    args.core_ref = "T13333N0.gene_model.protein,T4686N0.gene_model.protein,T4530N0.gene_model.protein,T3702N0.gene_model.protein,T35883N0.gene_model.protein"
    args.norm_ref = "T210225N0.gene_model.protein,T13216N0.gene_model.protein,T3414N0.gene_model.protein,T337451N0.gene_model.protein,T3435N0.gene_model.protein,T55571N0.gene_model.protein,T167602N0.gene_model.protein,T90708N0.gene_model.protein,T746888N0.gene_model.protein,T13894N0.gene_model.protein,T1510057N0.gene_model.protein,T51953N0.gene_model.protein,T42345N0.gene_model.protein,T4615N0.gene_model.protein,T15368N0.gene_model.protein,T1148796N0.gene_model.protein,T4577N0.gene_model.protein,T4639N0.gene_model.protein,T4641N0.gene_model.protein,T218851N0.gene_model.protein,T4432N0.gene_model.protein,T63787N0.gene_model.protein,T29760N0.gene_model.protein,T3694N0.gene_model.protein,T3847N0.gene_model.protein,T57918N0.gene_model.protein,T3659N0.gene_model.protein,T29730N0.gene_model.protein,T49390N0.gene_model.protein,T4155N0.gene_model.protein,T4146N0.gene_model.protein,T4182N0.gene_model.protein,T4072N0.gene_model.protein,T4081N0.gene_model.protein"
    args.ortho_level = 'orthogroups'
    args.orthofinder_wpgmapper_id_map = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/orthofinder_wpgmapper_id_map.txt'

    args.orthofinder_dir_path = "/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_ortho/protein/OrthoFinder/Results_Oct06"
    args.output_dir = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/Cau'
    args.tmp_path = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/Cau/tmp'
    args.log_file = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/Cau/log"

    args.top_evidence_num = 20
    args.tolerance_ratio = 0.80
    args.num_threads = 56
    args.annotated_coverage = 0.8
    args.query_in_mcl_cluster_ratio = 0.8
    args.support_threshold = 0.8
    args.jaccard_threshold = 0.8
    args.top_outgroup_num = 5

    args.min_cover = 0.5
    args.min_aa_len = 50
    args.min_identity = 0.3
    args.min_score = 50

    GeneLoss_main(args)


"""intervals
from https://ideone.com/JhYhNC

Union, intersection, set difference and symmetric difference
of possibly overlapping or touching integer intervals.
Intervals are defined right-open. (1, 4) -> 1, 2, 3

e.g.
union([(1, 4), (7, 9)], (3, 5)) -> [(1, 5), (7, 9)]
intersection([(1, 4), (7, 9)], (3, 5)) -> [(3, 4)]
set_difference([(1, 4), (7, 9)], (3, 5)) -> [(1, 3), (7, 9)]
set_difference([(3, 5)], [(1, 4), (7, 9)]) -> [(4, 5)]
symmetric_difference([(1, 4), (7, 9)], (3, 5)) -> [(1, 3), (4, 5), (7, 9)]

see: http://e...content-available-to-author-only...a.org/wiki/Set_theory#Basic_concepts_and_notation
"""

import copy
from collections import namedtuple
from itertools import accumulate, chain, islice, repeat
from operator import itemgetter
import unittest


class Intervals(object):
    """Holds a non overlapping list of intervals.
    One single interval is just a pair.
    Overlapping or touching intervals are automatically merged.
    """

    def __init__(self, interval_list=()):
        """Raises a ValueError if the length of one of the
        intervals in the list is negative.
        """
        if any(begin > end for begin, end in interval_list):
            raise ValueError('Invalid interval')
        self._interval_list = _merge_interval_lists(
            interval_list, [], OP_UNION)

    def __repr__(self):
        """Just write out all included intervals.
        """
        return 'Intervals ' + str(self._interval_list)

    def get(self, copy_content=True):
        """Return the list of intervals.
        """
        return copy.copy(self._interval_list) if copy_content \
            else self._interval_list


def union(a, b):
    """Merge a and b (union).
    """
    return Intervals(_merge_interval_lists(
        a.get(False), b.get(False), OP_UNION))


def intersections(a, b):
    """Intersects a and b.
    """
    return Intervals(_merge_interval_lists(
        a.get(False), b.get(False), merge_type=OP_INTERSECTIONS))


def set_difference(a, b):
    """Removes b from a.
    Set difference is not commutative.
    """
    return Intervals(_merge_interval_lists(
        a.get(False), b.get(False), merge_type=OP_SET_DIFFERENCE))


def symmetric_difference(a, b):
    """Symmetric difference of a and b.
    """
    return Intervals(_merge_interval_lists(
        a.get(False), b.get(False), merge_type=OP_SYMMETRIC_DIFFERENCE))


def compose(func_1, func_2, unpack=False):
    """
    compose(func_1, func_2, unpack=False) -> function

    The function returned by compose is a composition of func_1 and func_2.
    That is, compose(func_1, func_2)(5) == func_1(func_2(5)).
    """
    if not callable(func_1):
        raise TypeError("First argument to compose must be callable.")
    if not callable(func_2):
        raise TypeError("Second argument to compose must be callable.")

    if unpack:
        def composition(*args, **kwargs):
            return func_1(*func_2(*args, **kwargs))
    else:
        def composition(*args, **kwargs):
            return func_1(func_2(*args, **kwargs))
    return composition


# Helper to avoid repetitive lambdas.
def check_for(cond):
    return lambda val: val == cond


# http://e...content-available-to-author-only...a.org/wiki/Identity_function
identity = lambda x: x

# Depending on the operation carried out,
# the criteria for interval begins and ends in the result differ.
# E.g:
#      a = | - - - - |      | - - - |
#      b =     | - - - - |
# counts = 1   2     1   0  1       0   (union, intersection, sym diff)
# counts = 1   0    -1   0  1       0   (set diff)
# union  = | - - - - - - |  | - - - |
# inter  =     | - - - - |
# sym d  =           | - |
# set d  = | - |            | - - - |
#
# One can see that union begins if the count changes from 0 to 1
# and ends if the count changes from 1 to 0
# An intersection begins at a change from 1 to 2 and ends with 2 to 1.
# A symmetric difference begins at every change to one
# and ends at every change away from one.
# The conditions for the set difference are the same as for the union.
set_op = namedtuple('set_op', ['transform', 'begin', 'end'])

OP_UNION = set_op(
    transform=identity,
    begin=check_for((0, 1)),
    end=check_for((1, 0)))

OP_INTERSECTIONS = set_op(
    transform=identity,
    begin=check_for((1, 2)),
    end=check_for((2, 1)))

OP_SYMMETRIC_DIFFERENCE = set_op(
    transform=identity,
    begin=lambda change: change[1] == 1,
    end=lambda change: change[1] != 1)

OP_SET_DIFFERENCE = set_op(
    # If we want to calculate the set difference
    # we invert the second interval list,
    # i.e. swap begin and end.
    transform=compose(tuple, reversed),
    begin=check_for((0, 1)),
    end=check_for((1, 0)))


# class Intervals makes sure, by always building the union first,
# that no invalid a's or b's are fed here.
def _merge_interval_lists(a, b, merge_type):
    """Merges two lists of intervals in O(n*log(n)).
    Overlapping or touching intervals are simplified to one.

    Arguments:
    a and b -- The interval lists to merge.
    merge_type -- Can be:
                  OP_UNION,
                  OP_INTERSECTIONS,
                  OP_SYMMETRIC_DIFFERENCE, or
                  OP_SET_DIFFERENCE.

    Return the sorted result as a list.
    """

    # Adjust the operand for the selected operator.
    b = map(merge_type.transform, b)

    # Separately sort begins and ends
    # and pair them with the implied change
    # of the count of currently open intervals.
    # e.g. (1, 4), (7, 9), (3, 5) ->
    #     begins = [(1, 1), (3, 1), (7, 1)]
    #     ends = [(4, -1), (5, -1), (9, -1)]
    both = list(chain(a, b))
    begins = zip(sorted(map(itemgetter(0), both)),
                 repeat(1))
    ends = zip(sorted(map(itemgetter(1), both)),
               repeat(-1))

    # Sort begins and ends together.
    # If the value is the same, begins come before ends
    # to ensure touching intervals being merged to one.
    # In our example above this means:
    # edges = [(1, 4), (3, 1), (4, -1), (5, -1), (7, 1), (9, -1)]
    edges = sorted(chain(begins, ends), key=lambda x: (x[0], -x[1]))

    # The number of opened intervals after each edge.
    counts = list(accumulate(map(itemgetter(1), edges)))
    # The changes of opened intervals at each edge.
    changes = zip(chain([0], counts), counts)
    # Just the x positions of the edges.
    xs = map(itemgetter(0), edges)
    xs_and_changes = list(zip(xs, changes))

    # Now we filter out the begins and ends from the changes
    # and get their x positions.
    res_begins = map(itemgetter(0),
                     starfilter(lambda x, change: merge_type.begin(change),
                                xs_and_changes))
    res_ends = map(itemgetter(0),
                   starfilter(lambda x, change: merge_type.end(change),
                              xs_and_changes))

    # The result is then just pairing up the sorted begins and ends.
    result = pairwise(sorted(chain(res_begins, res_ends)), False)

    # No empty intervals in the result.
    def length_greater_than_zero(interval):
        return interval[0] < interval[1]

    return list(filter(length_greater_than_zero, result))


class TestIntervals(unittest.TestCase):

    def test_ctor(self):
        # Check ctors sanity check.
        self.assertRaises(ValueError, Intervals, [(2, 4), (3, 1)])

    def test_add_behind(self):
        # Check adding right of the last interval.
        intervals = Intervals([(0, 2)])
        intervals = union(intervals, Intervals([(3, 4)]))
        self.assertEqual(intervals.get(), [(0, 2), (3, 4)])

    def test_add_in_front(self):
        # Check adding left to the first interval.
        intervals = Intervals([(3, 4)])
        intervals = union(intervals, Intervals([(1, 2)]))
        self.assertEqual(intervals.get(), [(1, 2), (3, 4)])

    def test_add_in_between(self):
        # Check adding between two intervals.
        intervals = Intervals([(1, 2)])
        intervals = union(intervals, Intervals([(6, 9)]))
        intervals = union(intervals, Intervals([(3, 5)]))
        self.assertEqual(intervals.get(), [(1, 2), (3, 5), (6, 9)])

    def test_add_touching(self):
        # Check adding a interval touching an existing one.
        intervals = Intervals([(1, 3)])
        intervals = union(intervals, Intervals([(3, 5)]))
        self.assertEqual(intervals.get(), [(1, 5)])

    def test_add_overlapping(self):
        # Check adding a interval overlapping an existing one.
        intervals = Intervals([(1, 4)])
        intervals = union(intervals, Intervals([(3, 5)]))
        self.assertEqual(intervals.get(), [(1, 5)])

    def test_add_overlapping_multiple(self):
        # Check adding a interval overlapping multiple existing ones.
        intervals = Intervals([(1, 4)])
        intervals = union(intervals, Intervals([(5, 7)]))
        intervals = union(intervals, Intervals([(8, 10)]))
        intervals = union(intervals, Intervals([(3, 9)]))
        self.assertEqual(intervals.get(), [(1, 10)])

    def test_add_swallow(self):
        # Check adding a interval completely covering an existing one.
        intervals = Intervals([(2, 3)])
        intervals = union(intervals, Intervals([(1, 4)]))
        self.assertEqual(intervals.get(), [(1, 4)])

    def test_sub(self):
        # Check removing an interval
        intervals = Intervals([(0, 3)])
        intervals = union(intervals, Intervals([(5, 7)]))
        intervals = set_difference(intervals, Intervals([(2, 6)]))
        self.assertEqual(intervals.get(), [(0, 2), (6, 7)])

    def test_intersections(self):
        # Check adding right of the last interval.
        intervals = Intervals([(0, 3)])
        intervals = union(intervals, Intervals([(5, 7)]))
        intervals = intersections(intervals, Intervals([(2, 6)]))
        self.assertEqual(intervals.get(), [(2, 3), (5, 6)])

    def test_symmetric_difference(self):
        # Check symmetric difference
        intervals = Intervals([(0, 3)])
        intervals = union(intervals, Intervals([(5, 7)]))
        intervals = symmetric_difference(intervals, Intervals([(2, 6)]))
        self.assertEqual(intervals.get(), [(0, 2), (3, 5), (6, 7)])


def tuple_wise(iterable, size, step):
    """Tuples up the elements of iterable.

    Arguments:
    iterable -- source data
    size -- size of the destination tuples
    step -- step to do in iterable per destination tuple

    tuple_wise(s, 3, 1): "s -> (s0,s1,s2), (s1,s2,s3), (s3,s4,s5), ...
    tuple_wise(s, 2, 4): "s -> (s0,s1), (s4,s5), (s8,s9), ...
    """
    return zip(
        *(islice(iterable, start, None, step)
          for start in range(size)))


def pairwise(iterable, overlapping):
    """Pairs up the elements of iterable.
    overlapping: "s -> (s0,s1), (s2,s3), (s4,s5), ...
    not overlapping: "s -> (s0,s1), (s1,s2), (s2,s3), ...
    """
    return tuple_wise(iterable, 2, 1 if overlapping else 2)


def starfilter(function, iterable):
    """starfilter <--> filter  ==  starmap <--> map"""
    return (item for item in iterable if function(*item))


if __name__ == '__main__':
    unittest.main(verbosity=2)



def get_versions():
    return versions[0]["number"]


versions = [
    {
        "number": "0.0.1",
        "features": [
            "1. Separate the tools, libraries and api parts from the original Genome_work_tools and become ToolBiox",
        ],
    },
]


from datetime import date
import calendar
from Bio.Data import CodonTable

# Time
day_name = list(calendar.day_name)
day_abbr = list(calendar.day_abbr)

month_name = list(calendar.month_name)
month_abbr = list(calendar.month_abbr)

# codon table




if __name__ == '__main__':

    # get today
    print(date.today())

    # print a calendar for 2019
    print(calendar.calendar(2019))

    #



from Bio import SeqIO
from lib.xuyuxing.evolution.taxonomy import read_tax_name_record_dict_db

mirbase_path = "/lustre/home/xuyuxing/Database/mirbase/mirbase22/"
tax_record_dict_file = "/lustre/home/xuyuxing/Database/genome/info/NCBI/taxonomy/tax_record_dict.db"
miRNA_dat = mirbase_path + "miRNA.dat"

def find_miRNA_in_lineage():

    Viridiplantae_taxid = "33090"
    output_file = "/lustre/home/xuyuxing/Database/mirbase/mirbase22/plant.miRNA"

    num = 0
    output_list = []
    for record in SeqIO.parse(miRNA_dat, "embl"):
        mirna_id = record.id
        description = record.description
        species = description.split(" ")[0] + " " + description.split(" ")[1]
        for i in record.features:
            if i.type == "miRNA":
                product = i.qualifiers['product'][0]
                accession = i.qualifiers['accession'][0]
                evidence = i.qualifiers['evidence'][0]
                output_list.append((mirna_id, accession, product, evidence, species))
                num = num + 1

    output_list = list(set(output_list))

    output_dict = {}
    for mirna_id, accession, product, evidence, species in output_list:
        if accession in output_dict:
            if output_dict[accession]["evidence"] == "experimental":
                continue
        output_dict[accession] = {}
        output_dict[accession] = {
            "mirna_id": mirna_id,
            "accession": accession,
            "product": product,
            "evidence": evidence,
            "species": species
        }

    tax_name_list = [output_dict[i]["species"] for i in output_dict]
    tax_name_list = list(set(tax_name_list))

    tax_record_dict = read_tax_name_record_dict_db(tax_record_dict_file, tax_name_list)

    plant_miRNA = {}
    missed_taxid = []
    for i in output_dict:
        record = output_dict[i]
        if not record["species"] in tax_record_dict:
            missed_taxid.append(record["species"])
            continue
        record_tax_record = tax_record_dict[record["species"]]
        if hasattr(record_tax_record,"kingdom") and record_tax_record.kingdom == Viridiplantae_taxid:
            plant_miRNA[i] = output_dict[i]

    missed_taxid = list(set(missed_taxid))

    with open(output_file,"w") as f:
        for i in plant_miRNA:
            key_name = ["mirna_id", "accession", "product", "evidence", "species"]
            printer = ""
            for j in key_name:
                printer = printer+plant_miRNA[i][j]+"\t"
            printer.rstrip("\t")
            f.write(printer+"\n")


def get_all_ref():
    with open("/lustre/home/xuyuxing/Database/mirbase/mirbase22/refernces.txt","w") as f:
        output_list=[]
        for record in SeqIO.parse(miRNA_dat, "embl"):
            if "references" in record.annotations:
                for ref in record.annotations["references"]:
                    output_list.append((ref.title,ref.journal,ref.pubmed_id))
        output_list = list(set(output_list))
        for i in output_list:
            f.write("%s\t%s\t%s\n" % i)
